@@ -16,13 +16,16 @@ switch problem
  case 'pedcrank'
   level0=0; level=30;  alpha=10; 
   %movie_dir='/tmp/pedcrank/visual/';
-  movie_dir='../../build/visual/pedcrank2_eval_hist_0_30_10.0_overlap/';
-  histfile=sprintf('pedcrank2_eval_hist_%d_%d_%f.mat',...
+  %movie_dir='../../build/visual/pedcrank2_eval_hist_0_30_10.0_overlap/';
+  movie_dir='/LOCAL/rschneid/tmp/pedcrank/visual/';
+  %histfile=sprintf('pedcrank2_eval_hist_%d_%d_%f.mat',...
+  %    level0,level,alpha);
+  histfile=sprintf('../pedcrank2_eval_hist_%d_%d_%f.mat',...
       level0,level,alpha);
   plot_title=sprintf('pedcrank history, alpha=%7.1e',alpha);
   movie_basename=sprintf('pedcrank2_movie_%d_%d_%f_',...
       level0,level,alpha);
-  nmax=200;
+  nmax=400;
  otherwise
   error('swtich for unknown problem')
 end
