@@ -48,11 +48,13 @@ alpha=10.0;
 % should stay fixed, as only level is used for the histfiles !!!!!!!!!
 %level0=0; level=0; ref_stop=1e+3; ref_vx_max=1000;
 %level0=0; level=15; ref_stop=1e+2; ref_vx_max=10000;
-level0=0; level=30; ref_stop=1e+1; ref_vx_max=100000;
-%level0=0; level=45; ref_stop=1e+0; ref_vx_max=1000000;
+%level0=0; level=30; ref_stop=1e+1; ref_vx_max=100000;
+rpar_ini =[-1.3000;0.3000;0.7415;2.7780;1.8288;0.8290;5.2878;2.1769;2.2622;-0.8769;8.7567;1.5824;0.9068;0.2824;11.3000;-1.1374;1.8574;0.0100;3.1331;2.1967;0.3000;-0.1336;7.8171;-0.7146];
+level0=0; level=45; ref_stop=1e+0; ref_vx_max=1000000;
 %level0=0; level=50; ref_stop=1e-1; ref_vx_max=1500000;
 [crit0,dcrit0]=pedcrank2_eval([]);
 % pedcrank_constr.m also fits pedcrank2, using only Crpar,drpar
+rpar0=rpar_ini;
 [rpar,fevals,hist]=donlp2(rpar0,@pedcrank2_eval,@pedcrank_constr);
 
 %level0=0; level=35;
