@@ -402,7 +402,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
       rhs->V[i]=0.0;
     }
   *maxPe=0.0;
-  } /* END OPM master */
+  } /* END OMP master */
 
   /* clear rhsdot */
   if (rhsdot!=NULL)
@@ -413,7 +413,7 @@ int assem_conv_diff_tx(struct sparse *M, struct sparse *K,
 	{
 	  rhsdot->V[i]=0.0;
 	}
-      }  /* END OPM master */
+      }  /* END OMP master */
       dotfxc=&dotfxc_mem;
     }
   else
