@@ -53,7 +53,13 @@ enum elemtype {
 
 /* basic data structure for the mesh */
 struct mesh {
-  FIDX dim;           /* space dimension (2 for 2d, 3 for 3d) */
+  int  dim;           /* space dimension (2 for 2d, 3 for 3d) */
+  int type;           /* type of elements
+                         1=  P1 triangles
+                         2=  P2 triangles
+			 31= P1 tetrahedra
+			 32= P2 tetrahedra (p-hierarchical basis)
+		      */
   FIDX problem;       /* identifies the problem,
                          1= Poison Equation
                          2= Stokes flow
@@ -61,6 +67,12 @@ struct mesh {
                          4= Lame Equation (Linear Elasticity), 
                             in 2d plane strain
 		      */
+  int  curved;        /* in 2d ignored,
+			 in 3d: 
+			 0= not curved, polygonal domain
+			 1= curved, use gmsh +.geo to move points to geometry,
+			 2= curved, use .stl file to move points to geometry
+		      */
 
   FIDX vx_w;          /* number of double entries per vertex, in other
 			 words the width of a vertex entry   */
@@ -231,6 +243,21 @@ struct mesh {
 			 wrapper routines (e.g. to project new nodes
 			 to the geometry */
 
+  FIDX DD_proc_nr;    /* number of processors to be used (DD) */
+  FIDX DD_waves_nr;   /* number of waves of DD_proc_nr domains (DD) */
+  FIDX DD_nr;         /* number of subdomains in Domain Decomposition
+			 (DD), should be DD_proc_nr*DD_waves_nr */
+  FIDX *DD_elem_idx;  /* DD_elem_idx[i] index of start of subdomain i
+			 in DD_elem (length DD_nr+1) */
+  FIDX *DD_elem;      /* list of elements in each subdomain,
+			 DD_elem[DD_elem_idx[i]]
+			 ... DD_elem[DD_elem_idx[i+1]-1] */
+  FIDX *DD_node_idx;  /* DD_node_idx[i] index of start of subdomain i
+			 in DD_node (length DD_nr+1) */
+  FIDX *DD_node;      /* list of nodes in each subdomain,
+			 DD_node[DD_node_idx[i]]
+			 ... DD_node[DD_node_idx[i+1]-1] */
+
   /* detailed values for the positions of objects within the mesh
      arrays are element type dependent and therefore defined with text
      macros in the include file "meshdetails.h" */
@@ -375,6 +402,17 @@ struct mgdata {
 			 different levels */ 
 };
 
+/* structure to contain data for preconditioner for eigenvalue problems */
+struct eigP_data
+{
+  struct sparse *M;   /* the mass matrix */
+  struct sparse *A;   /* the stiffness matrix */
+  double lambda;      /* the eigenvalue under consideration */
+  struct vector hlp;  /* help vector for the multiplies */
+  struct mgdata *mg;  /* multigrid data structure */
+};
+
+
 
 /* structure to contain basis specific intergration data, these (or a
    vector of them should be attached to int_data, as userdat */
