@@ -308,7 +308,7 @@ int eval_basis(FIDX dim, enum elemtype type, FIDX subtype,
   else if (((type==tetra)&&(subtype==1))&&((dim==3)||(dim==2)))
     {
       /* numbering of the points in the masterelement:
-         (ttempted ascii art)
+         (attempted ascii art)
 
          3
 	 +_
@@ -359,7 +359,7 @@ int eval_basis(FIDX dim, enum elemtype type, FIDX subtype,
 	  /* let us define wsomething to make it more readable: */
 #define X (points[i*ldim +0])
 #define Y (points[i*ldim +1])
-#define Z (points[i*ldim +1])
+#define Z (points[i*ldim +2])
 	  /* the basis functions */
 	  (*phi)[i*(*num_basis)+0] = 1-X-Y-Z;
 	  (*phi)[i*(*num_basis)+1] = X;
@@ -388,6 +388,200 @@ int eval_basis(FIDX dim, enum elemtype type, FIDX subtype,
 	    (*hessphi)[i*(*num_basis)*ldim*ldim+j]=0.0;
 #undef X
 #undef Y
+#undef Z
+	}
+      return SUCCESS;
+    }
+  else if (((type==tetra)&&(subtype==2))&&((dim==3)||(dim==2)))
+    {
+      /* numbering of the points in the masterelement:
+         (attempted ascii art)
+
+         3
+	 +_
+	 |:\_
+	 | : \_
+	 | 9+  \_
+	 |   :   \_
+	7+   2+    +8_
+	 |   ;  "..  \_
+	 | 6+      "+. \_
+	 | ;       5  "..\_
+	 |;              ".\
+	0+---------+---------+1
+                   4
+
+
+         3
+	 +--__
+	 | \  --__9
+	 |   \_   +--___
+	 |     \_       --+ 2
+	 |       \_8   ..;|
+	7+         +_.;    |
+	 |      6+.; \_    |
+	 |    ..;      \_   +5
+	 | ..;           \_ |
+	 |;              ".\ |
+	0+---------+---------+ 1
+                   4
+
+	edge 0-1 is X-axis
+        edge 0-2 is Y-axis
+        edge 0-3 is Z-axis
+
+	the basis functions correspond to this numbering
+	basis functions 0..3 are linear nodal basis functions
+	                4..9 are quadratic edge bubble functions
+	4..9 computed with help of Matlab:
+	x=[0 0 0; 1 0 0; 0 1 0; 0 0 1; 0.5 0 0; 0.5 0.5 0; 0 0.5 0; 0 0 0.5; 0.5 0 0.5; 0 0.5 0.5];
+	plot3(x(:,1), x(:,2), x(:,3));
+	A=[0*x(:,1)+1, x(:,1), x(:,2), x(:,3),  x(:,1).^2, x(:,2).^2, x(:,3).^2,  x(:,1).*x(:,2), x(:,1).*x(:,3), x(:,2).*x(:,3)]
+	for i=5:10; b=zeros(10,1); b(i)=1; c=A\b; disp(sprintf('p%2d=%f+%f*x+%f*y+%f*z+%f*x^2+%f*y^2+%f*z^2+%f*xy+%f*xz+%f*yz\n',i-1,c)); end
+      */
+      int ldim=3;
+      *num_basis=10;
+
+      TRY_MALLOC( *phi, num_points*(*num_basis), double,
+		  eval_basis ); 
+      TRY_MALLOC( *gradphi, num_points*(*num_basis)*ldim, double,
+		  eval_basis );
+      TRY_MALLOC( *hessphi, num_points*(*num_basis)*ldim*ldim, double,
+		  eval_basis );
+      /* evaluate for each point */
+      for(i=0; i<num_points; i++)
+	{
+	  /* let us define wsomething to make it more readable: */
+#define X (points[i*ldim +0])
+#define Y (points[i*ldim +1])
+#define Z (points[i*ldim +2])
+	  /* the basis functions */
+	  /* linear nodal basis for nodes 0-3 */
+	  (*phi)[i*(*num_basis)+0] = 1-X-Y-Z;
+	  (*phi)[i*(*num_basis)+1] = X;
+	  (*phi)[i*(*num_basis)+2] = Y;
+	  (*phi)[i*(*num_basis)+3] = Z;
+	  /* quadratic edge bubbles for nodes 4-9 */
+	  (*phi)[i*(*num_basis)+4] = 4*X*(1-X-Y-Z);
+	  (*phi)[i*(*num_basis)+5] = 4*X*Y;
+	  (*phi)[i*(*num_basis)+6] = 4*Y*(1-X-Y-Z);
+	  (*phi)[i*(*num_basis)+7] = 4*Z*(1-X-Y-Z);
+	  (*phi)[i*(*num_basis)+8] = 4*X*Z;
+	  (*phi)[i*(*num_basis)+9] = 4*Y*Z;
+	  
+	  /* grad(phi_0) */
+	  (*gradphi)[i*(*num_basis)*ldim            ] = -1.0;
+	  (*gradphi)[i*(*num_basis)*ldim         + 1] = -1.0;
+	  (*gradphi)[i*(*num_basis)*ldim         + 2] = -1.0;
+	  /* grad(phi_1) */
+	  (*gradphi)[i*(*num_basis)*ldim+   ldim + 0] = 1.0;
+	  (*gradphi)[i*(*num_basis)*ldim+   ldim + 1] = 0.0;
+	  (*gradphi)[i*(*num_basis)*ldim+   ldim + 2] = 0.0;
+	  /* grad(phi_2) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 2*ldim + 0] = 0.0;
+	  (*gradphi)[i*(*num_basis)*ldim+ 2*ldim + 1] = 1.0;
+	  (*gradphi)[i*(*num_basis)*ldim+ 2*ldim + 2] = 0.0;
+	  /* grad(phi_3) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 3*ldim + 0] = 0.0;
+	  (*gradphi)[i*(*num_basis)*ldim+ 3*ldim + 1] = 0.0;
+	  (*gradphi)[i*(*num_basis)*ldim+ 3*ldim + 2] = 1.0;
+
+	  /* grad(phi_4) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 4*ldim + 0] =  4.0*(1.0-2.0*X-Y-Z);
+	  (*gradphi)[i*(*num_basis)*ldim+ 4*ldim + 1] = -4.0*X;
+	  (*gradphi)[i*(*num_basis)*ldim+ 4*ldim + 2] = -4.0*X;
+	  /* grad(phi_5) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 5*ldim + 0] = 4.0*Y;
+	  (*gradphi)[i*(*num_basis)*ldim+ 5*ldim + 1] = 4.0*X;
+	  (*gradphi)[i*(*num_basis)*ldim+ 5*ldim + 2] = 0.0;
+	  /* grad(phi_6) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 6*ldim + 0] = -4.0*Y;
+	  (*gradphi)[i*(*num_basis)*ldim+ 6*ldim + 1] =  4.0*(1.0-X-2.0*Y-Z);
+	  (*gradphi)[i*(*num_basis)*ldim+ 6*ldim + 2] = -4.0*Y;
+	  /* grad(phi_7) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 7*ldim + 0] = -4.0*Z;
+	  (*gradphi)[i*(*num_basis)*ldim+ 7*ldim + 1] = -4.0*Z;
+	  (*gradphi)[i*(*num_basis)*ldim+ 7*ldim + 2] =  4.0*(1.0-X-Y-2.0*Z);
+	  /* grad(phi_8) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 8*ldim + 0] = 4.0*Z;
+	  (*gradphi)[i*(*num_basis)*ldim+ 8*ldim + 1] = 0.0;
+	  (*gradphi)[i*(*num_basis)*ldim+ 8*ldim + 2] = 4.0*X;
+	  /* grad(phi_9) */
+	  (*gradphi)[i*(*num_basis)*ldim+ 9*ldim + 0] = 0.0;
+	  (*gradphi)[i*(*num_basis)*ldim+ 9*ldim + 1] = 4.0*Z;
+	  (*gradphi)[i*(*num_basis)*ldim+ 9*ldim + 2] = 4.0*Y;
+
+	  /* linear basis functions have vanishing Hessian */
+	  for (j=0; j<4*ldim*ldim; j++)
+	    (*hessphi)[i*(*num_basis)*ldim*ldim+j]=0.0;
+
+	  /* Hessian(phi_4) */
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim       + 0] = -8.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim       + 1] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim       + 2] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim+ ldim + 0] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim+ ldim + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim+ ldim + 2] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim+2*ldim+ 0] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim+2*ldim+ 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 4*ldim*ldim+2*ldim+ 2] = 0;
+
+	  /* Hessian(phi_5) */
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim       + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim       + 1] = 4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim       + 2] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim+ ldim + 0] = 4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim+ ldim + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim+ ldim + 2] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim+2*ldim+ 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim+2*ldim+ 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 5*ldim*ldim+2*ldim+ 2] = 0;
+
+	  /* Hessian(phi_6) */
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim       + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim       + 1] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim       + 2] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim+ ldim + 0] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim+ ldim + 1] = -8.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim+ ldim + 2] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim+2*ldim+ 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim+2*ldim+ 1] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 6*ldim*ldim+2*ldim+ 2] = 0;
+
+	  /* Hessian(phi_7) */
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim       + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim       + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim       + 2] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim+ ldim + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim+ ldim + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim+ ldim + 2] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim+2*ldim+ 0] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim+2*ldim+ 1] = -4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 7*ldim*ldim+2*ldim+ 2] = -8.0;
+
+	  /* Hessian(phi_8) */
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim       + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim       + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim       + 2] = 4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim+ ldim + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim+ ldim + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim+ ldim + 2] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim+2*ldim+ 0] = 4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim+2*ldim+ 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 8*ldim*ldim+2*ldim+ 2] = 0;
+
+	  /* Hessian(phi_9) */
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim       + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim       + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim       + 2] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim+ ldim + 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim+ ldim + 1] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim+ ldim + 2] = 4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim+2*ldim+ 0] = 0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim+2*ldim+ 1] = 4.0;
+	  (*hessphi)[i*(*num_basis)*ldim*ldim+ 9*ldim*ldim+2*ldim+ 2] = 0;
+#undef X
+#undef Y
 #undef Z
 	}
       return SUCCESS;
