@@ -38,6 +38,16 @@ void dgesv_( int* N, int* NRHS, double* A, int* LDA, int* IPIV,
 void dsyev_( char* JOBZ, char* UPLO, int* N, double* A, int* LDA, 
 	     double* W, double* WORK, int* LWORK, int* INFO );
 
+/* DSYGV( ITYPE, JOBZ, UPLO, N, A, LDA, B, LDB, W, WORK,
+          LWORK, INFO ) 
+   determine optimal LWORK by workspace querry: 
+      set LWORK=-1, run, WORK[0] will hold optimal LWORK (if INFO=0)
+*/
+void dsygv_( int* ITYPE, char* JOBZ, char* UPLO, int* N, 
+	     double* A, int* LDA, double* B, int* LDB, 
+	     double* W, double* WORK, int* LWORK, int* INFO );
+
+void dlarnv_( int* IDIST, int* ISEED, int* N, double* X );
 
 
 #endif
