@@ -72,10 +72,14 @@ struct timezone {
 
 
 /* define the integer datatypes, */
-/* FIDX is for indices into arrays/vectors, also general integer type */
-#define FIDX long 
-/* the corresponding format string (d for int, ld for long int...) */
+/* FIDX is for indices into arrays/vectors, also general integer type 
+   and the corresponding format string (d for int, ld for long int...) */
+/*#define FIDX long 
 #define dFIDX "ld"
+/* */
+#define FIDX int
+#define dFIDX "d" 
+/* */
 
 /* new adjoint handling, without replacing rows in the matrices */
 #define NEW_ADJOINT
