@@ -570,6 +570,388 @@ int gen_proj_MG_tx(void *arg1, struct vector *in,
   return SUCCESS;
 }
 
+
+
+
+/*FUNCTION*/
+int gen_eig_preco_MG_tx(void *arg1, struct vector *in,
+		   void *arg3, struct vector *out
+/* performs multigrid V-cycle for, 
+     (A-lambda*M)*out=in
+
+   where P projects the velocity components of boundary nodes to zero,
+   (such that addition of a projected vector doesn't change the
+   velocity there), and C^-1 repressents the action of the multigrid
+   preconditioning, that is one V cycle
+   
+   Input:  arg1    - not used, in the interface for compatibility reasons
+           in      - input vector
+	   arg3=
+	   eigP    - eigP data struct, containing everything needed for
+	             the matrix multiplies and multigrid
+
+   Output: out    - (given by reference), C^-1*in
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/
+		 ){
+  FIDX i, j, len;
+  int err;
+  FIDX dim, vx_nr;
+
+  struct vector xi, bi, invdiagi, dxi;
+
+  struct eigP_data *eigP;
+  struct sparse *M, *A;
+  double lambda;
+  struct mgdata *mg;
+
+  struct mesh *m;
+  struct multilvl *ml;
+
+  struct sparse *Ks;
+  struct projector1 *P;
+
+  double *xl, *dxl, *bl, *invdiag;
+
+  double alpha;
+
+  FIDX lvl, lmax, lmin, lvl_vx, smooths, vcycles, vccount;
+
+
+  eigP   = arg3;
+
+  A      = (*eigP).A;
+  M      = (*eigP).M;
+  lambda = (*eigP).lambda;
+  mg     = (*eigP).mg;
+
+  m      = (*mg).msh;
+  ml     = (*mg).ml;
+
+  Ks     = (*mg).Ks;
+  P      = (*mg).P;
+
+  dim    = (*ml).dim;
+  vx_nr  = (*mg).vx_nr;
+  len    = dim * vx_nr;
+
+  xl      = (*mg).xl;
+  dxl     = (*mg).dxl;
+  bl      = (*mg).bl;
+  invdiag = (*mg).invdiag;
+
+  smooths  = (*mg).smooths;
+  vcycles  = (*mg).vcycles;
+  alpha    = (*mg).CGC_scale;
+
+  if ((len!=(*in).len)||(len!=(*out).len))
+    {
+      fprintf(stderr,
+	      "gen_eig_preco_MG_tx: in or out length doesn't match\n");
+      return FAIL;
+    }
+
+
+  lmax = (*ml).lmax;
+
+  lmin=0; 
+
+
+  /* copy in to bl */
+  for (i=0; i<(*in).len; i++)
+    {
+      bl[i]=(*in).V[i];
+    }
+
+  /* apply P^T */
+  for (i=0; i<(*P).len; i++)
+    {
+      bl[ (*P).V[i] ] = 0.0 ;
+    }
+
+  /* apply C^-1 */
+
+  if (vcycles<=0)
+    {
+      fprintf(stderr, "gen_eig_preco_MG_tx: vcycles<=0 not defined\n");
+      return FAIL;
+    }
+ 
+
+  /* perform vcycles V-cycles */
+  for (vccount=0; vccount<vcycles; vccount++)
+    {
+      /* V-cycle downward */
+      for (lvl=lmax; lvl>=lmin; lvl--)
+	{ 
+	  lvl_vx=(*ml).nlevl[lvl]-(*ml).nlevl[lvl+1];
+	  
+	  if ((vccount==0)||(lvl<lmax))
+	    {
+	      /* set xl=0 */
+	      for (j=(*ml).nlevl[lvl+1]; j<(*ml).nlevl[lvl]; j++)
+		xl[j]=0.0;
+	    }
+	  
+	  /* make x_l, b_l, invdiag_l accessible as vector */
+	  xi.len=lvl_vx;
+	  xi.n_max=lvl_vx;
+	  xi.V=&xl[(*ml).nlevl[lvl+1]];
+
+	  bi.len=lvl_vx;
+	  bi.n_max=lvl_vx;
+	  bi.V=&bl[(*ml).nlevl[lvl+1]];
+
+	  invdiagi.len=lvl_vx;
+	  invdiagi.n_max=lvl_vx;
+	  invdiagi.V=&invdiag[(*ml).nlevl[lvl+1]];
+
+	  if ((lvl==0)&&((*mg).cmat!=NULL))
+	    {
+	      /* use the coarse grid solver */
+	      struct vector bcoarse;
+	      bcoarse.len=lvl_vx;
+	      bcoarse.n_max=lvl_vx;
+	      bcoarse.V=&bl[(*ml).nlevl[lvl+1]];
+
+	      /* projection */
+	      for (i=0; i<(*P).len; i++)
+		{
+		  FIDX dof, child;
+		  dof=(*P).V[i];
+		  MLVLFINDENTRY(child, dof, lvl, *ml);
+		  if (child>=0)
+		    {
+		      bl[child]=0.0;
+		    }
+		}
+
+
+	      err=coarse_mat_solve( (*mg).cmat, NoTrans,
+				    &bcoarse, &xi);
+	      FUNCTION_FAILURE_HANDLE( err, coarse_mat_solve,
+				       gen_eig_preco_MG_tx);    
+	    }
+	  else
+	    {
+	      /* do smooths Gauss-Seidel sweeps forward */
+	      if (lvl==lmax)
+		{
+		  /* eigenvalue special stuff on the finest level */
+		  for (i=0; i<len; i++)
+		    {
+		      double res_i, diag_i;
+		      double *entry;
+		      FIDX rows[1];
+		      rows[0]=i;
+
+		      /* res = b - (A -lambda*M) *x  */
+		      res_i=bi.V[i];
+
+		      err=sparse_row_tim_vec(A, 1, rows, 
+					     &xi, &(*eigP).hlp);
+		      FUNCTION_FAILURE_HANDLE( err, sparse_row_tim_vec,
+					       gen_eig_preco_MG_tx);
+		      res_i -= (*eigP).hlp.V[0];
+
+		      err=sparse_row_tim_vec(M, 1, rows, 
+					     &xi, &(*eigP).hlp);
+		      FUNCTION_FAILURE_HANDLE( err, sparse_row_tim_vec,
+					       gen_eig_preco_MG_tx);
+		      res_i += lambda* (*eigP).hlp.V[0];
+
+
+		      err=sparse_get_entry(A, i, i, 0, &entry);
+		      FUNCTION_FAILURE_HANDLE( err, sparse_get_entry,
+					       gen_eig_preco_MG_tx);
+		      diag_i = *entry;
+
+		      err=sparse_get_entry(M, i, i, 0, &entry);
+		      FUNCTION_FAILURE_HANDLE( err, sparse_get_entry,
+					       gen_eig_preco_MG_tx);
+		      diag_i -= lambda * (*entry); /* */
+
+		      xi.V[i] += res_i / diag_i;
+		    } /* end for i */
+		}
+	      else
+		{
+		  /* Gauss-Seidel with the Stiffness matrix on all
+		     other levels */
+		  err=sparse_GS_sweep_fwd( &Ks[lvl], &bi,
+					   &invdiagi, smooths, &xi);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_GS_sweep_fwd,
+					   gen_eig_preco_MG_tx);
+		}
+		    
+	      if (lvl==0)
+		{
+		  /* do smooths Gauss-Seidel sweeps backward, not done
+		   here, as symmetry of the preconditioner is less of
+		   an issue */
+		}
+	    }
+
+	  
+	  if (lvl>0)
+	    {
+	      /* compute the residual on this lvl */
+	      /* compute the matrix vector product,
+		 dxl=K*xl */
+	      dxi.V   = &dxl[(*ml).nlevl[lvl+1]];
+	      dxi.len = lvl_vx;
+	      dxi.n_max = lvl_vx;
+
+	      if (lvl==lmax)
+		{
+		  err=sparse_mul_mat_vec( M, &xi, &dxi);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_mul_mat_vec,
+					   gen_eig_preco_MG_tx);
+		  for (i=0; i<dxi.len; i++)
+		    {
+		      dxi.V[i] *= -lambda;
+		    }
+
+		  err=sparse_mul_mat_vec_add( A, &xi, &dxi);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_mul_mat_vec,
+					   gen_eig_preco_MG_tx);
+		  
+		  /* now change dxl to rl=bl-(A-lambda*M)*xl */
+		  for (j=(*ml).nlevl[lvl+1]; j<(*ml).nlevl[lvl]; j++)
+		    dxl[j]=bl[j]-dxl[j];
+		}
+	      else
+		{
+		  err=sparse_mul_mat_vec( &Ks[lvl], &xi, &dxi);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_mul_mat_vec,
+					   gen_eig_preco_MG_tx);
+
+		  /* now change dxl to rl=bl-K*xl */
+		  for (j=(*ml).nlevl[lvl+1]; j<(*ml).nlevl[lvl]; j++)
+		    dxl[j]=bl[j]-dxl[j];
+		}
+
+	      /* restrict rl to r_l-1 */
+	      err=mg_restrict_tx( mg, lvl, lvl-1, dxl);
+	      FUNCTION_FAILURE_HANDLE( err, mg_restrict_tx,
+				       gen_eig_preco_MG_tx);
+	      /* copy r_l-1 to b_l-1 */
+	      for (j=(*ml).nlevl[lvl]; j<(*ml).nlevl[lvl-1]; j++)
+		bl[j]=dxl[j];
+	    }
+	} /* end V-cycle downward */
+
+      /* V-cycle upward */
+      for (lvl=lmin+1; lvl<=lmax; lvl++)
+	{ 
+	  lvl_vx=(*ml).nlevl[lvl]-(*ml).nlevl[lvl+1];
+
+	  /* apply the update computed in the lower level */
+	  /* copy the update to dx */
+	  for (j=(*ml).nlevl[lvl]; j<(*ml).nlevl[lvl-1]; j++)
+	      dxl[j]=xl[j];
+
+
+	  /* interpolate dx to lvl */
+	  err=mg_interpolate_tx( mg, lvl-1, lvl, dxl);
+	  FUNCTION_FAILURE_HANDLE( err, mg_interpolate_tx,
+				   gen_eig_preco_MG_tx);
+
+
+	  /* apply the update to xl */
+	  for (j=(*ml).nlevl[lvl+1]; j<(*ml).nlevl[lvl]; j++)
+	    xl[j]+=alpha*dxl[j];
+
+	  /* make x_l, b_l, invdiag_l accessible as vector */
+	  xi.len=lvl_vx;
+	  xi.n_max=lvl_vx;
+	  xi.V=&xl[(*ml).nlevl[lvl+1]];
+
+	  bi.len=lvl_vx;
+	  bi.n_max=lvl_vx;
+	  bi.V=&bl[(*ml).nlevl[lvl+1]];
+
+	  invdiagi.len=lvl_vx;
+	  invdiagi.n_max=lvl_vx;
+	  invdiagi.V=&invdiag[(*ml).nlevl[lvl+1]];
+	  
+	  /* do smooths Gauss-Seidel sweeps backward */
+	  if (lvl==lmax)
+	    {
+	      /* eigenvalue special stuff on the finest level */
+	      for (i=len-1; i>=0; i--)
+		{
+		  double res_i, diag_i;
+		  double *entry;
+		  FIDX rows[1];
+		  rows[0]=i;
+		      
+		  /* res = b - (A -lambda*M) *x  */
+		  res_i=bi.V[i];
+
+		  err=sparse_row_tim_vec(A, 1, rows, 
+					 &xi, &(*eigP).hlp);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_row_tim_vec,
+					   gen_eig_preco_MG_tx);
+		  res_i -= (*eigP).hlp.V[0];
+		  
+		  err=sparse_row_tim_vec(M, 1, rows, 
+					 &xi, &(*eigP).hlp);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_row_tim_vec,
+					   gen_eig_preco_MG_tx);
+		  res_i += lambda* (*eigP).hlp.V[0];
+		  
+
+		  err=sparse_get_entry(A, i, i, 0, &entry);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_get_entry,
+					   gen_eig_preco_MG_tx);
+		  diag_i = *entry;
+		  
+		  err=sparse_get_entry(M, i, i, 0, &entry);
+		  FUNCTION_FAILURE_HANDLE( err, sparse_get_entry,
+					   gen_eig_preco_MG_tx);
+		  diag_i -= lambda * (*entry); /* */
+		  
+		  xi.V[i] += res_i / diag_i;
+		} /* end for i */
+	    }
+	  else
+	    {
+	      err=sparse_GS_sweep_bwd( &Ks[lvl], &bi,
+				       &invdiagi, smooths, &xi);
+	      FUNCTION_FAILURE_HANDLE( err, sparse_GS_sweep_bwd,
+				       gen_eig_preco_MG_tx);
+	    }
+	} /* end V-cycle upward */
+    } /* end V-cycles loop */
+  
+  (*mg).vccount+=vccount;
+
+
+
+  /* copy xl to out */
+  for (i=0; i<(*out).len; i++)
+    {
+      (*out).V[i]=xl[i];
+    }
+
+  /* apply P */
+  for (i=0; i<(*P).len; i++)
+    {
+      (*out).V[ (*P).V[i] ] = 0.0 ;
+    }
+
+  /* done */
+
+  return SUCCESS;
+}
+
+
+
+
+
 /* inline functions */
 #include "assembly_inline_funs.h"
 
@@ -593,9 +975,10 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
    but for now it is only a simplified version of this idea.
 
    
-   Input:  m       - the mesh, has to be T1 or T2 
+   Input:  m       - the mesh, has to be T1, T2, E1, or E2
            type    - defining which type of mesh this actually is
 	             type=1 ==> linear,  type=2 quadratic triangles
+	             type=31 ==> linear,  type=32 quadratic tetrahedra
            ncomp   - number of (solution) components in u, i.e. number
                      of variables per node in the mesh
 	   u       - coefficient vector representing a function on
@@ -617,7 +1000,7 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 */
 		 ){
   FIDX i, j, r, el, pt, cmp;
-  int err;
+  int err, subtype;
 
   FIDX dim, basn, bas_n, vx_nr, el_w, vx_w;
 
@@ -632,10 +1015,20 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 			  0.5, 0.0,
 			  0.5, 0.5,
 			  0.0, 0.5 };
-  double points_e1[4*4]={ 0.0, 0.0, 0.0,
+  double points_e1[3*4]={ 0.0, 0.0, 0.0,
 			  1.0, 0.0, 0.0,
 			  0.0, 1.0, 0.0,
 			  0.0, 0.0, 1.0 };
+  double points_e2[3*10]={ 0.0, 0.0, 0.0,
+			   1.0, 0.0, 0.0,
+			   0.0, 1.0, 0.0,
+			   0.0, 0.0, 1.0,
+			   0.5, 0.0, 0.0,
+			   0.5, 0.5, 0.0,
+			   0.0, 0.5, 0.0,
+			   0.0, 0.0, 0.5,
+			   0.5, 0.0, 0.5,
+			   0.0, 0.5, 0.5  };
   double *points;
 
 
@@ -683,27 +1076,12 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
   switch (type)
     {
     case 1:
-      switch (dim)
-	{
-	case 2:
-	  l_mctxVXSTRT = MCT1VXSTRT;
-	  l_mctxELNOD1 = MCT1ELNOD1;
-	  points = points_t1;
-	  basn   = 3;
-	  eltype = tria;
-	  break;
-	case 3:
-	  l_mctxVXSTRT = MCE1VXSTRT;
-	  l_mctxELNOD1 = MCE1ELNOD1;
-	  points = points_e1;
-	  basn   = 4;
-	  eltype = tetra;
-	  break;
-	default:
-	  fprintf(stderr,"gen_zz_interpolate_gradient_tx: unknown dim=%d\n",
-		  dim);
-	  return FAIL;
-	}
+      l_mctxVXSTRT = MCT1VXSTRT;
+      l_mctxELNOD1 = MCT1ELNOD1;
+      points = points_t1;
+      basn   = 3;
+      eltype = tria;
+      subtype = 1;
       break;
     case 2:
       l_mctxVXSTRT = MCT2VXSTRT;
@@ -711,6 +1089,23 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
       points = points_t2;
       basn   = 6;
       eltype = tria;
+      subtype = 2;
+      break;
+    case 31:
+      l_mctxVXSTRT = MCE1VXSTRT;
+      l_mctxELNOD1 = MCE1ELNOD1;
+      points = points_e1;
+      basn   = 4;
+      eltype = tetra;
+      subtype = 1;
+      break;
+    case 32:
+      l_mctxVXSTRT = MCE1VXSTRT;
+      l_mctxELNOD1 = MCE1ELNOD1;
+      points = points_e2;
+      basn   = 10;
+      eltype = tetra;
+      subtype = 2;
       break;
     default:
       fprintf(stderr,"gen_zz_interpolate_gradient_tx: unknown type=%d\n",type);
@@ -719,7 +1114,7 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 
   /* init */
   /* initialise interpolation info  */
-  err= eval_basis( dim, eltype, type, basn, points,
+  err= eval_basis( dim, eltype, subtype, basn, points,
 		   &bas_n, &phi, &gradp, &hessphi);
   FUNCTION_FAILURE_HANDLE( err, eval_basis, gen_zz_interpolate_gradient_tx);
   free( hessphi );
@@ -842,6 +1237,27 @@ int gen_zz_interpolate_gradient_tx(struct mesh *m,
 	  gradu->V[r*ncomp*vx_nr + cmp*vx_nr +i]/=weight_count[i];
 	}
 
+  if (type==32)
+    {
+      /* change nodes to hierarchical where necessary */
+      for (i=0; i<(*m).eg_nr; i++)
+	{
+	  if ( (*m).edge[i*(*m).eg_w+MCE1EGCHL1]==-1 )
+	    {
+	      FIDX nodm=(*m).edge[i*(*m).eg_w+MCE2EGNODM];
+	      FIDX nod0=(*m).edge[i*(*m).eg_w+MCE1EGNOD1  ];
+	      FIDX nod1=(*m).edge[i*(*m).eg_w+MCE1EGNOD1+1];
+	      for(r=0; r<dim; r++)
+		for(cmp=0; cmp<ncomp; cmp++)
+		  {
+		    gradu->V[r*ncomp*vx_nr + cmp*vx_nr +nodm]
+		      -= 0.5*(gradu->V[r*ncomp*vx_nr + cmp*vx_nr +nod0]
+			      + gradu->V[r*ncomp*vx_nr + cmp*vx_nr +nod1]);
+		  }
+	    }
+	}
+    }
+
   free(lgradu);
   free(ldofs);
   free(Jinvgrad);
@@ -941,38 +1357,32 @@ int gen_error_est_ZZ_tx(struct mesh *m, FIDX ncomp, struct vector *uh,
   switch (type)
     {
     case 1:
-      switch (dim)
-	{
-	case 2:
-	  l_mctxVXSTRT = MCT1VXSTRT;
-	  l_mctxELNOD1 = MCT1ELNOD1;
-	  eltype = tria;
-	  order=4;
-	  break;
-	case 3:
-	  l_mctxVXSTRT = MCE1VXSTRT;
-	  l_mctxELNOD1 = MCE1ELNOD1;
-	  eltype = tetra;
-	  order = 2;
-	  break;
-	default:
-	  fprintf( stderr,
-		   "gen_error_est_ZZ_tx: dim=%d is not supported\n",
-		   dim);
-	  return FAIL;
-	}
+      l_mctxVXSTRT = MCT1VXSTRT;
+      l_mctxELNOD1 = MCT1ELNOD1;
+      eltype = tria;
+      order=4;
+      subtypes=1;
       break;
     case 2:
       l_mctxVXSTRT = MCT2VXSTRT;
       l_mctxELNOD1 = MCT2ELNOD1;
       eltype = tria;
       order=7;
-      if (dim!=2)
-	{
-	  fprintf( stderr,
-		   "gen_error_est_ZZ_tx type=2 is only implemented for 2D meshes!\n");
-	  return FAIL;
-	}			    
+      subtypes=2;
+      break;
+    case 31:
+      l_mctxVXSTRT = MCE1VXSTRT;
+      l_mctxELNOD1 = MCE1ELNOD1;
+      eltype = tetra;
+      order = 2;
+      subtypes=1;
+      break;
+    case 32:
+      l_mctxVXSTRT = MCE1VXSTRT;
+      l_mctxELNOD1 = MCE1ELNOD1;
+      eltype = tetra;
+      order = 6;
+      subtypes=2;
       break;
     default:
       fprintf(stderr,"gen_error_est_ZZ_tx: unknown type=%d\n",type);
@@ -988,7 +1398,6 @@ int gen_error_est_ZZ_tx(struct mesh *m, FIDX ncomp, struct vector *uh,
   FUNCTION_FAILURE_HANDLE( err, gen_zz_interpolate_gradient_tx, gen_error_est_ZZ_tx);
   
 
-  subtypes=type;
 
   err=cubature_bases( dim, order, eltype, 1, &subtypes, &iform); 
   FUNCTION_FAILURE_HANDLE( err, cubature_bases, gen_error_est_ZZ_tx);
@@ -1094,7 +1503,7 @@ int gen_error_est_ZZ_tx(struct mesh *m, FIDX ncomp, struct vector *uh,
       /* printf("el=%3d   nk=%e\n", el, nk.V[el]); */
     } /* end loop over elements */
 
-  /* Finally, decide which elements are to marked */      
+  /* Finally, decide which elements are to be marked */      
   printf("gen_error_est_ZZ_tx->");
   err=gen_error_est_marker(m, &nk, marker, nrmarked, globest, set);
   FUNCTION_FAILURE_HANDLE( err, gen_error_est_marker, gen_error_est_ZZ_tx);
