@@ -87,6 +87,28 @@ int gen_proj_MG_tx(void *arg1, struct vector *in,
            FAIL    - failure, see error message
 */);
 
+int gen_eig_preco_MG_tx(void *arg1, struct vector *in,
+		   void *arg3, struct vector *out
+/* performs multigrid V-cycle for, 
+     (A-lambda*M)*out=in
+
+   where P projects the velocity components of boundary nodes to zero,
+   (such that addition of a projected vector doesn't change the
+   velocity there), and C^-1 repressents the action of the multigrid
+   preconditioning, that is one V cycle
+   
+   Input:  arg1    - not used, in the interface for compatibility reasons
+           in      - input vector
+	   arg3=
+	   eigP    - eigP data struct, containing everything needed for
+	             the matrix multiplies and multigrid
+
+   Output: out    - (given by reference), C^-1*in
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
 int gen_zz_interpolate_gradient_tx(struct mesh *m,
 				   int type,
 				   FIDX ncomp,
