@@ -104,6 +104,12 @@ int lame_dIdu_t2( struct mesh *m,
   FIDX  bas_n1d1, bd;
   FIDX edi, *node, fu;
   int assem_el, assem_bd;
+
+  double crit_14_inner_integral=0.0, crit_14_inner_area=0.0; 
+  double crit_14_outer_integral=0.0, crit_14_outer_area=0.0; 
+
+  double crit_14_inner_avg_sig[3]={0.0, 0.0, 0.0};
+  double crit_14_outer_avg_sig[3]={0.0, 0.0, 0.0};
   
   /* init */
   /* get integration formulas */
@@ -172,6 +178,9 @@ int lame_dIdu_t2( struct mesh *m,
 	    break;
 	  case 13: /* displacement in given point */
 	    break;
+	  case 14: /* target stress tensor in stripe */
+	    assem_el=1;
+	    break;
 	  default: /* unknown type for Lame */
 	    fprintf(stderr,"lame_dIdu_t2: "
 	      "unknown pccrit type=%"dFIDX"\n",type);
@@ -354,6 +363,65 @@ int lame_dIdu_t2( struct mesh *m,
 			      }
 			}
 		      break;
+		    case 14: /* target stress tensor in stripe */
+		      dhelp=iform.weights[k]*AdetJac;
+		      { 
+			double eps_11, eps_12, eps_22;
+			double sig_11, sig_12, sig_22;
+			double Emod, nu;
+
+			Emod= mu*(3.0*lambda+2.0*mu)/(lambda+mu);
+			nu=  lambda/(2.0*(lambda+mu));
+
+			eps_11= gradu[0*dim+0];
+			eps_12= 0.5*(gradu[0*dim+1]+gradu[1*dim+0]);
+			eps_22= gradu[1*dim+1];
+
+			sig_11= Emod/( (1.0+nu)*(1.0-2.0*nu) )
+			  *( (1.0-nu)*eps_11 + nu*eps_22 );
+			sig_22= Emod/( (1.0+nu)*(1.0-2.0*nu) )
+			  *( nu*eps_11 + (1.0-nu)*eps_22 );
+			sig_12= Emod/(1.0+nu)*eps_12;
+
+
+			int inner=
+			  ( (*m).pcvol[(*m).face[el*fc_w+MCT2FCPCVL]*(*m).pv_w
+				       +MCXXPVCRIT      ]==0 );
+			if (inner==1)
+			  {
+			    double diff_sig_11, diff_sig_12, diff_sig_22;
+
+			    diff_sig_11= sig_11-(*m).pccrit[crit*(*m).pc_w+
+							   MCXXPCDAT1+2];
+			    diff_sig_22= sig_22-(*m).pccrit[crit*(*m).pc_w+
+							   MCXXPCDAT1+3];
+			    diff_sig_12= sig_12-(*m).pccrit[crit*(*m).pc_w+
+							   MCXXPCDAT1+4];
+
+			    crit_14_inner_area += dhelp;
+			    crit_14_inner_integral += dhelp*
+			      (diff_sig_11*diff_sig_11 +
+			       diff_sig_22*diff_sig_22 +
+			       diff_sig_12*diff_sig_12);
+
+			    crit_14_inner_avg_sig[0]+= dhelp*sig_11;
+			    crit_14_inner_avg_sig[1]+= dhelp*sig_22;
+			    crit_14_inner_avg_sig[2]+= dhelp*sig_12;
+			  }
+			else
+			  {
+			    crit_14_outer_area += dhelp;
+			    crit_14_outer_integral += dhelp*
+			      (sig_11*sig_11 +
+			       sig_22*sig_22 +
+			       sig_12*sig_12);
+
+			    crit_14_outer_avg_sig[0]+= dhelp*sig_11;
+			    crit_14_outer_avg_sig[1]+= dhelp*sig_22;
+			    crit_14_outer_avg_sig[2]+= dhelp*sig_12;
+			  }
+		      }
+		      break;
 		    default:
 		      break;
 		    }
@@ -367,26 +435,52 @@ int lame_dIdu_t2( struct mesh *m,
      over elements or edges*/
   for (crit=0; crit<(*m).pc_nr; crit++)
     {
-      /* square of deformation in set of points */
-      if ( (*m).pccrit[crit*(*m).pc_w+MCXXPCTYPE] == 13 )
+      type=(FIDX)(*m).pccrit[crit*(*m).pc_w+MCXXPCTYPE];
+      switch(type)
 	{
-	  double u_rj;
-	  r=(int) (*m).pccrit[crit*(*m).pc_w+MCXXPCDAT1];
-	  dhelp=(*m).pccrit[crit*(*m).pc_w+MCXXPCDAT1+1];
-	  for (j=0; j<dim; j++)
-	    {
-	      u_rj= (*u).V[j*vx_nr+r];       
-	      if (Is != NULL)
-		{
-		  *Is += u_rj*u_rj*dhelp;
-		}
-	      if (dIdu != NULL)
-		{
-		  (*dIdu).V[j*vx_nr+r]+=2*u_rj*dhelp;
-		}
-	    }
+	case 13: /* square of deformation in set of points */
+	  {
+	    double u_rj;
+	    r=(int) (*m).pccrit[crit*(*m).pc_w+MCXXPCDAT1];
+	    dhelp=(*m).pccrit[crit*(*m).pc_w+MCXXPCDAT1+1];
+	    for (j=0; j<dim; j++)
+	      {
+		u_rj= (*u).V[j*vx_nr+r];       
+		if (Is != NULL)
+		  {
+		    *Is += u_rj*u_rj*dhelp;
+		  }
+		if (dIdu != NULL)
+		  {
+		    (*dIdu).V[j*vx_nr+r]+=2*u_rj*dhelp;
+		  }
+	      }
+	  }
+	  break;
+	case 14:
+	  fprintf(stderr,"lame_dIdu_t2 DEBUG: "
+		  "crit_14_inner_integral=%9.2e  crit_14_inner_area=%9.2e  "
+		  "crit_14_outer_integral=%9.2e  crit_14_outer_area=%9.2e\n",
+		  crit_14_inner_integral, crit_14_inner_area,
+		  crit_14_outer_integral, crit_14_outer_area);
+	  fprintf(stderr,"lame_dIdu_t2 DEBUG: "
+		  "crit_14_inner_sig: %9.2e  %9.2e  %9.2e\n",
+		  crit_14_inner_avg_sig[0]/crit_14_inner_area,
+		  crit_14_inner_avg_sig[1]/crit_14_inner_area,
+		  crit_14_inner_avg_sig[2]/crit_14_inner_area);
+	  fprintf(stderr,"lame_dIdu_t2 DEBUG: "
+		  "crit_14_outer_sig: %9.2e  %9.2e  %9.2e\n",
+		  crit_14_outer_avg_sig[0]/crit_14_outer_area,
+		  crit_14_outer_avg_sig[1]/crit_14_outer_area,
+		  crit_14_outer_avg_sig[2]/crit_14_outer_area);
+
+	  *Is= crit_14_inner_integral/crit_14_inner_area
+	    +  1e0*crit_14_outer_integral/crit_14_outer_area;
+	  break;
 	}
     } 
+
+  
     
   /* free local data */
   free_intdata (&iform1);
@@ -395,6 +489,7 @@ int lame_dIdu_t2( struct mesh *m,
   free(gradu);
   free(Jacinv);
   free(Jinvgrad);
+  free(node);
     
   return SUCCESS;
 }
@@ -2373,27 +2468,27 @@ int lame_eppler_conjecture_2010_rem3_3( struct mesh *m,
     cheat_sigma.len=dim*vx_nr;
 
     cheat_sigma.V=&sigmu.V[0];
-    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+    err=mesh_write_solution_vtk_tx( m, &cheat_sigma, dim, NULL, 0,
 				    25, "visual/eppler_sigma_u__1" );
-    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			     lame_eppler_conjecture_2010_rem3_3);
 
     cheat_sigma.V=&sigmu.V[dim*vx_nr];
-    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+    err=mesh_write_solution_vtk_tx( m, &cheat_sigma, dim, NULL, 0,
 				    25, "visual/eppler_sigma_u__2" );
-    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			     lame_eppler_conjecture_2010_rem3_3);
 
 
     cheat_sigma.V=&sig_gamma.V[0];
-    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+    err=mesh_write_solution_vtk_tx( m, &cheat_sigma, dim, NULL, 0,
 				    31, "visual/eppler_sigma_gamma_u__1" );
-    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			     lame_eppler_conjecture_2010_rem3_3);
     cheat_sigma.V=&sig_gamma.V[dim*vx_nr];
-    err=mesh_write_solution_vtk_t2( m, &cheat_sigma, dim, NULL, 0,
+    err=mesh_write_solution_vtk_tx( m, &cheat_sigma, dim, NULL, 0,
 				    31, "visual/eppler_sigma_gamma_u__2" );
-    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+    FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			     lame_eppler_conjecture_2010_rem3_3);
   }/*/
   
