@@ -371,12 +371,6 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
                      as defined by the projector P are left unchanged,
                      the rest is set to the solution of the system
                      under the Dirichlet conditions 
-	   K       - Stiffness matrix, rows and columns corresponding
-                     to Dirichlet DOFs (see projector P) are modified
-                     (for simplicity of implementation),
-		     K must have symmetric connectivity structure, but
-		     entries may be non-symmetric, ideally K should be
-		     still in flexible storage format,
 	   rhs     - right-hand-side vector, is modified to implement
                      Dirichlet conditions (for simplicity of
                      implementation) 
@@ -390,6 +384,9 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
 
 
    Input:  P       - projector defining the Dirichlet DOFs,
+	   K       - Stiffness matrix, 
+		     K must have symmetric connectivity structure, but
+		     entries may be non-symmetric
            task    - specify task:
                      task==1 => initial solve, umfpackdata, isdiri and
                                 help are created
@@ -404,8 +401,9 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
 			 ){
   FIDX ndofs, i;
   int err;
+  FIDX   *tmp_cols;
+  double *tmp_A;
 
-  ndofs=x->len;
 
 
   if (task==9)
@@ -421,6 +419,7 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
       return SUCCESS;
     }
 
+  ndofs=x->len;
 
   if (task==1)
     { 
@@ -444,24 +443,28 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
 
   /* the remaining stuff is only task==1 and task==2 */
 
-  /* set all non-Dirichlet components in x to zero */
-  for (i=0; i<ndofs; i++) 
+  /* if K is still provided, modify rhs to account for Dirichlet data */
+  if (K!=NULL)
     {
-      if ((*isdiri)[i]==0)
+      /* set all non-Dirichlet components in x to zero */
+      for (i=0; i<ndofs; i++) 
 	{
-	  x->V[i]=0.0;
+	  if ((*isdiri)[i]==0)
+	    {
+	      x->V[i]=0.0;
+	    }
 	}
-    }
 
+      /* multiply K*x, and compute rhs=rhs-K*x */
+      err=sparse_mul_mat_vec( K, x, help);
+      FUNCTION_FAILURE_HANDLE( err, sparse_mul_mat_vec,
+			       sparse_solve_UMFPACK_reuse);  
 
-  /* multiply K*x, and compute rhs=rhs-K*x */
-  err=sparse_mul_mat_vec( K, x, help);
-  FUNCTION_FAILURE_HANDLE( err, sparse_mul_mat_vec, sparse_solve_UMFPACK_reuse);  
-
-  for (i=0; i<ndofs; i++)
-    {
-      rhs->V[i] -= help->V[i];
-    }
+      for (i=0; i<ndofs; i++)
+	{
+	  rhs->V[i] -= help->V[i];
+	}
+    } /* end if K!=NULL */
 
   /* project rhs values at dirichlet dofs */
   for (i=0; i<P->len; i++)
@@ -481,6 +484,27 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
 	  "try to use type FLEX\n\n\n");
 	  }
 	  /* */
+      /* copy initial data of K to temporary arrays, to allow 
+	 modification and restoration after that */
+      if ((*K).type != SP_TYPE_COMPROW)
+	{
+	  fprintf(stderr,"sparse_solve_UMFPACK_reuse: "
+		  "assume COMPROW here "
+		  "-> error\n");
+	  return FAIL;
+	}
+      {
+	FIDX nrdata=(*K).rows[(*K).row_nr];
+	TRY_MALLOC(tmp_cols, nrdata, FIDX, sparse_solve_UMFPACK_reuse);
+	TRY_MALLOC(tmp_A, nrdata, double, sparse_solve_UMFPACK_reuse);
+	for (i=0; i<nrdata; i++)
+	  {
+	    tmp_cols[i]= (*K).cols[i];
+	    tmp_A[i]= (*K).A[i];
+	  }
+      }
+
+
       for (i=0; i<P->len; i++)
 	{
 	  FIDX dirdof= P->V[i];
@@ -491,45 +515,37 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
 	    {
 	    case SP_TYPE_FLEX:
 	      {
-		struct int_double_list *curr_col, *curr_ofdiag, *pre;
+		FIDX j;
+
+		fprintf(stderr,"sparse_solve_UMFPACK_reuse: "
+			"FLEX not repaired after last change of FLEX format "
+			"-> error\n");
+		return FAIL;
 	    
-		/* delete the of-diagonal-entries of the column first, 
-		   using the row (flex_rows[row] is the diagonal entry) */ 
-		curr_col = (*K).flex_rows[dirdof].succ;
-		while (curr_col != NULL)
+		/* set the of-diagonal-entries of the column to zero first, 
+		   using the row 
+		   (flex_row_cols[row][0] is the diagonal entry) */ 
+		/*for (j=1; j<(*K).flex_row_col_nr[dirdof]; j++)
 		  {
 		    FIDX col;
+		    double *entry;
 		
-		    col = curr_col->ix;
+		    col = (*K).flex_row_cols[dirdof][j];
 		    /* delete the dirdof column in row col */
-		    pre= &((*K).flex_rows[col]);
-		    curr_ofdiag = curr_ofdiag->succ;
-		    while ((curr_ofdiag !=NULL) && (curr_ofdiag->ix != dirdof))
-		      {
-			pre         = curr_ofdiag;
-			curr_ofdiag = curr_ofdiag->succ;
-		      }
-		    if (curr_ofdiag != NULL)
-		      {
-			/* delete entry */
-			pre->succ = curr_ofdiag->succ;
-			free(curr_ofdiag);
-		      }
-		    else
+		    /*err=sparse_get_entry( K, col, dirdof, 0, &entry);
+		    if (err=FAIL)
 		      {
 			fprintf(stderr,"sparse_solve_UMFPACK_reuse: (FLEX)"
 				"connectivity of matrix not symmetric -> error\n");
 			return FAIL;
 		      }
-		
-		    /* next column in row dirdof */
-		    curr_col = curr_col->succ;
+		    (*entry)=0;
 		  }
 		/* now delete of-diagonals of the row itself */
-		int_double_list_free( (*K).flex_rows[dirdof].succ );
-		(*K).flex_rows[dirdof].succ = NULL;
+		/*(*K).flex_row_col_nr[dirdof] = 1;
 		/* set diagonal entry to unity */
-		(*K).flex_rows[dirdof].dx   = 1.0;
+		/*(*K).flex_row_cols[dirdof][0] = dirdof;
+		  (*K).flex_row_data[dirdof][0] = 1.0;*/
 	      }
 	      break;
 	    case SP_TYPE_COMPROW:
@@ -627,7 +643,20 @@ int sparse_solve_UMFPACK_reuse(struct vector *x,
       FUNCTION_FAILURE_HANDLE( err, linsolver_init, sparse_solve_UMFPACK_reuse);  
       err=UMFPACK_factorize (K, umfpackdata );
       FUNCTION_FAILURE_HANDLE( err, UMFPACK_factorize, sparse_solve_UMFPACK_reuse);  
-    }
+
+      /* restore K to undo modifications */
+      {
+	FIDX nrdata=(*K).rows[(*K).row_nr];
+	for (i=0; i<nrdata; i++)
+	  {
+	    (*K).cols[i] = tmp_cols[i];
+	    (*K).A[i] = tmp_A[i];
+	  }
+	free(tmp_A);
+	free(tmp_cols);
+      }
+
+    } /* end if task==1 */
 
   /* solve with the factors of K, solution in help */
   err=UMFPACK_solve(umfpackdata, rhs, help);
