@@ -143,7 +143,7 @@ int mesh_write_solution_exp_t1( struct mesh *m, struct vector *x,
                      valid
 */);
 
-int mesh_write_solution_vtk_t1( struct mesh *m, struct vector *vec_x,
+int mesh_write_solution_vtk_tx( struct mesh *m, struct vector *vec_x,
 				FIDX vcompo, struct vector *scalar_x,
 				FIDX scompo,
 				FIDX namlen, char *name
@@ -184,6 +184,30 @@ int mesh_write_solution_vtk_t1( struct mesh *m, struct vector *vec_x,
                      valid
 */);
 
+int mesh_write_solution_vtk_warp_tx( struct mesh *m, struct vector *scalar_x,
+				     FIDX namlen, char *name,
+				     double *retMinData, double *retMaxData
+/* writes the mesh and the solution into a vtk readable
+   file for visualisation, 
+   this can for example be used with paraview
+           http://www.paraview.org/
+
+   Input:  m         - the mesh
+           scalar_x  - vector with data associated to the nodes,
+	               for representing 1 scalar field,
+	   namlen    - maximal useable length of name
+	   name      - basename of the file, full name will be
+	               <name>.femp
+
+   Output: (writes the file)
+           retMinData - the minimum of the scalar_x (by reference)
+           retMaxData - the maximum of the scalar_x (by reference)
+
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/);
+
 int mesh_write_solution_femplot_t1( struct mesh *m, struct vector *x,
 				FIDX ncompo,
 				FIDX namlen, char *name
@@ -469,7 +493,7 @@ int mesh_read_boundary_t1( FILE *in, struct mesh *m, FIDX bd_nr, FIDX sg_nr
 */			 );
 
 int mesh_read_triangle_wrapper( FILE *infile, struct mesh *m,
-				FIDX hole_nr
+				FIDX hole_nr, FIDX rhsf
 /* reads the hole part of a mesh file, then calls the mesh generator
    "triangle" to generate a mesh (mainly the elements, plus changes of
    the boundary), for a descrition of the file format, see the example
@@ -481,6 +505,7 @@ int mesh_read_triangle_wrapper( FILE *infile, struct mesh *m,
    input:   infile  - a file open for reading
             hole_nr - the number of hole entries in the file, as
                       given by the header
+            rhsf    - the index of the rhs function, set -1 to ignore
    
    output:  m       - (given by reference) a mesh, the vertex,
                       element, edge and bound parts are reallocated
@@ -579,6 +604,22 @@ int mesh_read_function( FILE *in, struct mesh *m, FIDX fu_nr
             FAIL    - failure, see error message 
 */			 );
 
+int mesh_read_rhsf( FILE *in, struct mesh *m, FIDX *rhsf
+/* reads the rhsf part of a mesh file, if any,
+   for a descrition of the file
+   format, see the example file "mesh_example.f1m"
+
+   input:   in      - a file open for reading
+   
+   output:  m       - (given by reference) a mesh, the function part is
+                      allocated and defined
+            rhsf    - specified function, if one is specified in the mesh file, 
+	              -1 otherwise (ignore indicator)
+
+   retun:   SUCCESS - success,
+            FAIL    - failure, see error message 
+*/			 );
+
 int mesh_read_parameter( FILE *in, struct mesh *m, FIDX pa_nr
 /* reads the parameter part of a mesh file, for a descrition of the file
    format, see the example file "mesh_example.f1m"
@@ -638,6 +679,38 @@ int mesh_read_compute_conns_t1( struct mesh *m
             FAIL    - failure, see error message 
 */			 );
 
+int mesh_pcvol_compute_stripe(struct mesh *m,
+			      FIDX crit_inner, FIDX crit_outer,
+			      FIDX bd1, FIDX bd2
+/* re-defines the performance criteria for all elements, based on
+   whether they are between shape segments bd1 and bd2, or outside of
+   those: inside the criterion is changed to crit_inner, outside to crit_outer
+   
+   IN/OUT:   m         - the mesh, pcvol is modified accordingly
+   
+   INPUT:    crit_inner- id of the pccrit entry for the elements between 
+                         bd1 and bd2
+	     crit_outer- id of the pccrit entry for the elements outside of 
+	                 bd1 and bd2
+             bd1
+	     bd2       - shape segments that enclose the inner part of 
+	                 the criterion
+ 
+   RETURN:   SUCCESS   - success
+	     FAIL      - failure, see error message, output will not be valid
+
+*/);
+
+int mesh_init_DD_elem( struct mesh *m
+/* initialises/sets the DD_elem part of the mesh
+
+   in/out:  m       - (given by reference) a mesh, the DD_* parts are
+                      initialised or set 
+
+   retun:   SUCCESS - success,
+            FAIL    - failure, see error message 
+*/			 );
+
 int mesh_refine_uniform_t2( struct mesh *m 
 /* refines all elements of the mesh m, all necessarry new parts are
    created thereby, increases mesh.lvl
@@ -843,7 +916,8 @@ int mg_restrict_t2( struct mgdata *mg,
    In/Out: mg      - multigrid data, the done part is used and therby
                      overwritten, other parts may be used read only
                      (phih)
-	   mlv     - a multilevel vector, the lvlin part is read, and
+	   mlv     - a multilevel vector, 
+	             the lvlin part is read and modified, and
 	             all levels lvl with lvlin>lvl>=lvlout are
 	             overwritten by the restriction from lvlin
 
@@ -933,7 +1007,8 @@ int mg_restrict_t1( struct mgdata *mg, FIDX lvlin, FIDX lvlout, double *mlv
    In/Out: mg      - multigrid data, the done part is used and therby
                      overwritten, other parts may be used read only
                      (phih)
-	   mlv     - a multilevel vector, the lvlin part is read, and
+	   mlv     - a multilevel vector,
+	             the lvlin part is read and modified, and
 	             all levels lvl with lvlin>lvl>=lvlout are
 	             overwritten by the restriction from lvlin
 
@@ -1099,15 +1174,6 @@ int mesh_write_solution_femplot_t2( struct mesh *m, struct vector *x,
                      valid
 */);
 
-int mesh_write_solution_vtk_t2( struct mesh *m,
-				struct vector *vec_x, FIDX vcompo,
-				struct vector *scalar_x, FIDX scompo,
-				FIDX namlen, char *name
-/* converts mesh to type t1 and uses mesh_write_solution_vtk_t1, see
-   there for description,
-   mesh m has to be of type t2
-*/);
-
 int mesh_t1_to_t2( struct mesh *m1, struct mesh *m2
 /* creates the T2 mesh by converting the T1 mesh m1 into a T2 mesh,
    Input:  m1         - T1 mesh
@@ -1180,6 +1246,12 @@ int mesh_sseg_adjust_tx( struct mesh *m, int insert,
                      curved boundary, ==1 may instert, ==0 not,
 		     ==1 only usefull if in the pre-mesh-generator
 		     stage, as this could destroy the connectivity
+		     the nodes are distributed such that the
+		     t-values such that |x_i - x_(i-1)| <= 4*c_curve*r, 
+		     where r is the local curvature radius of the spline,
+		     c_curve << 0.5    (better <=1/10) 
+		     and t_i-t_(i-1) <= delta_t_max
+
 	   nI      - number of performance criteria, size of dIdF and
                      dIdx, ignored if dIdF==NULL
            dIdx    - derivatives of performance criteria wrt node
@@ -1239,6 +1311,72 @@ int mesh_sseg_adjust_nodes( struct mesh *m, FIDX seg, FIDX seg_nds,
                      valid
 */);
 
+int mesh_sseg_build_tmesh_t1(double **tmesh, FIDX *nt,
+			     struct mesh *m, FIDX seg,
+			     double t_0, double t_end, 
+			     double c_curve, double delta_t_max
+/* creates a mesh of t-values 
+
+      t_0<t_1<t_2...<t_(nt-1) 
+
+   such that 
+
+      |x_i - x_(i-1)| <= 4*c_curve*r,
+   and 
+      t_i-t_(i-1) <= delta_t_max
+
+   where r is the local curvature radius of the spline, 
+
+   Let d be the distance between the spline and the polygon of edges
+   that will result from the t-mesh. The above constraint guarantees
+      d < c_curve*|x_i - x_(i-1)|
+   if 
+      c_curve << 0.5 (better <= 1/10) 
+   
+
+   Output: tmesh   - (pointer to double given by reference)
+                     vector of length nt, containig t_0,...,t_(nt-1),
+                     memory is allocated here, has to be freed again
+		     by the calling routine, (pointer give by reference)
+           nt      - (FIDX given by reverence)
+	             the number of mesh points
+
+   Input:  m       - the mesh,
+           seg     - id of the segment
+	   t_0     - starting point of the t-mesh
+	   t_end   - end point of the t-mesh
+	   c_curve - constant for the constraint, see above
+           delta_t_max
+                   - bound on distance of successive t-values
+
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/);
+
+int mesh_sseg_tmesh_constraintfunc_t1(double *ftk,
+				      struct mesh *m, FIDX seg, 
+				      double tk, double c_curve
+/* evaluates the constraint function 
+
+     "f(tk)=4*c_curve*r(tk)", 
+  
+   at the given t-value tk, where r(t) is the local curvature radius
+   of the spline
+   
+
+   OUTPUT: ftk	   - value of the constraint function f(tk)
+
+   INPUT:  seg     - the shape segment
+	   m	   - the mesh 
+	   tk	   - t-value for the bezier spline
+           c_curve - constant as described above
+			 
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/);
+
 int mesh_more_vertices ( struct mesh *m, FIDX need
 /* increases the number of vertices that can be held in the mesh by
    reallocating memory
@@ -1480,8 +1618,8 @@ int mesh_spline_set_t( struct mesh *m, FIDX seg, FIDX n1, FIDX n2, FIDX nm,
    adjusted
 
    INPUT/Output:
-	    m		- the mesh:m.st[midnode] and 
-	                  (optional) m.vertex[midonde] will be set
+	    m		- the mesh: m.st[nm] and 
+	                  (optional) m.vertex[nm] will be set
    INPUT:   seg	        - the shape segment
 	    n1,
 	    n2     	- start and end node of the edge under consideration
