@@ -205,16 +205,38 @@ int mesh_gmsh_project_nodes_e1( struct mesh *m,
 		      in edgeVL and faceVL are modified
 
    input:  edgeVL   - list of new nodes that have a designated EDGE 
-	              geometry number, two list elements per node:
-	              first the node number, second the geometry tag/id,
+	              geometry number, three list elements per node:
+	              first the node number, second the geometry tag/id, 
+		      third an edge ID (for computation of characteristic 
+		      length)
 	   faceVL   - list of new nodes that have a designated FACE
-	              geometry number, two list elements per node:
+	              geometry number, three list elements per node:
 	              first the node number, second the geometry tag/id,
+		      third an edge ID (for computation of characteristic 
+		      length)
 
    return:  SUCCESS - success,
             FAIL    - failure, see error message 
 */);
 
+int mesh_switch_type_e1( struct mesh *m, int type
+/* modifies the mesh so that all elements are of the polynomial degree
+   given by type, all necessarry new parts are created thereby,
+   
+
+   In/Out: m       - the mesh, is modified accordingly, e.g.
+                     elements, volumes, faces, edges, vertices, are modified,
+		     if necessary the fields of the mesh are
+		     reallocated to have space for the new stuff
+
+   Input:  type    - type==31 for P1 tetrahedra
+                     type==32 for P2 tetrahedra
+
+   Return: SUCCESS - success
+           FAIL    - failure, see error message, output will not be
+                     valid
+*/);
+
 int mesh_refine_uniform_e1( struct mesh *m 
 /* refines all elements of the mesh m, all necessarry new parts are
    created thereby, increases mesh.lvl
@@ -251,25 +273,41 @@ int mesh_refine_element_e1( struct mesh *m , FIDX el,
 		      reallocated to have space for the new stuff
 	   tail_GEVL- list of new nodes that have a designated EDGE 
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 	   tail_GFVL- list of new nodes that have a designated FACE
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
                      valid
 */);
 
+void mesh_elem_reorder( FIDX *elem, int *order, FIDX *tmp, int nr
+/* reorders the entries of elem according to order
+
+   Input:  order    - array of nr indices (permutation), 
+                      the new order of the old entries of elem
+	   nr       - size of elem, order, and tmp
+
+   In/Out: elem     - array to be reordered,
+                      elem[j]=elem[order[j]]
+           tmp      - help array
+*/);
+
 int mesh_refine_element_e1_innerels( struct mesh *m, FIDX oldvol,
 				     FIDX *faces1, FIDX *faces2, 
-				     FIDX *newvols
+				     FIDX *newvols, FIDX *oct_diag_edge
 /* the faces1 and faces2 together describe an octahedron, 
    for this the shortest diagonal is found, and introduced as new 
    edge, four new faces (triangles) are created with this diagonal
@@ -291,6 +329,8 @@ int mesh_refine_element_e1_innerels( struct mesh *m, FIDX oldvol,
    Output: newvols  - array of 3 volume indices, the ones which are
                       created new, the fourth new volume overwrites
                       the original volume vo
+           oct_diag_edge
+                    - index of the diagonal edge used in the above
 
 
    Return: SUCCESS - success
@@ -323,7 +363,7 @@ int mesh_split_face_e1( struct mesh *m , FIDX fc,
 
    Input:  fc       - the face to be refined
            voldnod  - array of 4 node indices, to which nodnewface and
-                      nodopsnewedge
+                      nodopsnewedge correspond
 
    In/Out: m        - the mesh, is modified accordingly, e.g. new
                       faces, edges, vertices, boundary
@@ -345,16 +385,20 @@ int mesh_split_face_e1( struct mesh *m , FIDX fc,
                       first still available of (0,1,2)
 	   tail_GEVL- list of new nodes that have a designated EDGE 
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 	   tail_GFVL- list of new nodes that have a designated FACE
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 
    Output: newcenterface
                     - the central face of the four new faces created
@@ -385,10 +429,12 @@ int mesh_split_edge_e1( struct mesh *m , FIDX eg,
 		      reallocated to have space for the new stuff
 	   tail_GEVL- list of new nodes that have a designated EDGE 
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
@@ -417,16 +463,20 @@ int mesh_split_face_green_e1( struct mesh *m , FIDX fc,
 		      reallocated to have space for the new stuff
 	   tail_GEVL- list of new nodes that have a designated EDGE 
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 	   tail_GFVL- list of new nodes that have a designated FACE
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 
    Return: SUCCESS - success
            FAIL    - failure, see error message, output will not be
@@ -455,16 +505,20 @@ int mesh_refine_element_green_e1( struct mesh *m , FIDX el,
 		      reallocated to have space for the new stuff
 	   tail_GEVL- list of new nodes that have a designated EDGE 
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 	   tail_GFVL- list of new nodes that have a designated FACE
 	              geometry number, if an node is produced during
-	              refinement, then two list elements are appended:
+	              refinement, then three list elements are appended:
 	              first the node number, second the geometry tag/id,
 		      later this list should be used to move the nodes
-		      onto their geometry
+		      onto their geometry,
+		      third an edge ID (for computation of characteristic 
+		      length)
 
 
    Return: SUCCESS - success
