@@ -145,7 +145,8 @@ int mesh_deform_t1_t2(struct mesh *msh1, FIDX lvlm,
     {
       vx_nr=msh2.vx_nr; 
 
-      err=sparse_flex_alloc( &Ks_linel[lvl], dim*vx_nr);
+      err=sparse_flex_alloc( &Ks_linel[lvl], dim*vx_nr,
+			     dim*FEINS_SPARSE_COLS_P_ROW_T2);
       FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, mesh_deform_t1_t2);
 
       err=assem_lame_tx( &Ks_linel[lvl], NULL, &msh2, NULL, NULL,
@@ -160,7 +161,8 @@ int mesh_deform_t1_t2(struct mesh *msh1, FIDX lvlm,
   vx_nr=msh2.vx_nr; 
 
 
-  err=sparse_flex_alloc( &Ks_linel[lvlm], dim*vx_nr);
+  err=sparse_flex_alloc( &Ks_linel[lvlm], dim*vx_nr, 
+			 dim*FEINS_SPARSE_COLS_P_ROW_T2);
   FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, mesh_deform_t1_t2);
 
   err=projector1_alloc( &P_linel, dim*vx_nr );
@@ -596,8 +598,8 @@ int mesh_deform_start_t2(struct mesh *m, struct sparse *K
   err=projector1_alloc( &P, dim*vx_nr );
   FUNCTION_FAILURE_HANDLE( err, projector1_alloc, mesh_deform_start_t2);
   
-  err=sparse_flex_alloc( &K[lvl], dim*vx_nr);
-  FUNCTION_FAILURE_HANDLE( err, sparse_alloc, mesh_deform_start_t2); 
+  err=sparse_flex_alloc( &K[lvl], dim*vx_nr, dim*FEINS_SPARSE_COLS_P_ROW_T2);
+  FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, mesh_deform_start_t2); 
   
   err=assem_lame_tx_tx( &K[lvl], &P, m, NULL, NULL,  
 			 lambda, mu, 0, 1, 2);
