@@ -95,6 +95,10 @@
 /*                                                                  */
 /********************************************************************/
 
+/* elements */
+#define MCXXELDDPR 0              /* position inside element, where
+				     subdomain index is stored */
+
 /*   pcsurf */
 #define MCXXPSSURF 0              /* corresponding surface element,
 				     edge in 2d, face in 3d */
@@ -214,6 +218,21 @@
 					 coef   - a coefficient by
 					          which the result is 
 						  multiplied
+                                     == 14.0 (Lame-Problems)
+				        data= bd1, bd2, 
+					      sigma11, sigma22, sigma12
+                                        bd1, bd2 - the stripe of elements
+					           between shape segments 
+						   bd1 and bd2 is Omega_D
+					sigma11,sigma22,sigma12
+                                                 - target stress tensor inside 
+						   Omega_D,
+						 - outside of Omega_D 
+						   (in Omega_R=Omega\Omega_D)
+						   the target is zero stress
+						 - performance is 
+					   1/|Omega_D|*(|sigma-sigma_D|_F)
+					   +1/|Omega_R|*(|sigma|_F)
 				  */
 #define MCXXPCDAT1 (MCXXPCTYPE+1) /* begin of data */
 #define MCXXPCDATX 6              /* maximal length of a data block */
@@ -348,7 +367,7 @@
 #define MCT1VXLN   (MCT1VXSTRT+2) /* length of one vertex entry */
 
 /*   elements */
-#define MCT1ELNOD1 0              /* first node of the element   */
+#define MCT1ELNOD1 MCXXELDDPR+1   /* first node of the element   */
 #define MCT1ELLN   (MCT1ELNOD1+3) /* length of one element entry */
 
 /*   edges */
@@ -436,7 +455,7 @@
 #define MCT2VXLN   (MCT2VXSTRT+2) /* length of one vertex entry */
 
 /*   elements */
-#define MCT2ELNOD1 0              /* first node of the element,
+#define MCT2ELNOD1 MCXXELDDPR+1   /* first node of the element,
 				     6 nodes per element, numbered as:
 
 				     2
@@ -450,7 +469,9 @@
 /*   edges */
 #define MCT2EGNOD1 0              /* first node of the edge, there are
 				     2 nodes + */
-#define MCT2EGNODM (MCT2EGNOD1+2) /* the midnode of the edge, 1 entry */
+#define MCT2EGNODM (MCT2EGNOD1+2) /* the midnode of the edge, 1 entry
+				     NODM has to be directly after NOD1+1
+				  */
 #define MCT2EGCHL1 (MCT2EGNODM+1) /* child data, =-1 <=> no childs,
 				     other <=> they are childs (there
 				     are 2 childs) */
@@ -652,19 +673,28 @@
 #define MCE1VXSTRT 0              /* begin of component values, there
 				     are 3 components */
 #define MCE1VXLN   (MCE1VXSTRT+3) /* length of one vertex entry */
+#define MCEXVXTYPE (MCE1VXLN)     /* type of the vertex entry:
+				     ==0.0 coordinates of node 
+				     ==2.0 hierarchical update information
+				           from linear to quadratic
+					   interpolation
+				     only used if mesh.type>=32 and
+				     vx_w >= MCEXVXTYPE
+				  */
 
 /*   elements */
-#define MCE1ELNOD1 0              /* first node of the element   */
-#define MCE1ELLN   (MCE1ELNOD1+4) /* length of one element entry */
+#define MCE1ELNOD1 MCXXELDDPR+1   /* first node of the element   */
+#define MCE1ELLN   (MCE1ELNOD1+4) /* length of one P1 element entry,
+				     if other type of element is used, 
+				     el_w takes precedence over ELLN
+				   */
 
 
 
 /*   edges */
-#define MCE1EGNOD1 0              /* first node of the edge, there are
-				     2 nodes */
-#define MCE1EGCHL1 (MCE1EGNOD1+2) /* child data, =-1 <=> no childs,
-				     other <=> they are childs (there
-				     are 2 childs) */
+#define MCE1EGCHL1 0              /* child data, =-1 <=> no childs,
+				     other <=> they are children (there
+				     are 2 children) */
 
 #define MCE1EGGEO  (MCE1EGCHL1+2) /* index of geometry info, which is
 				     to be observed for this edge
@@ -672,7 +702,12 @@
 				  */
 #define MCE1EGLVL  (MCE1EGGEO+1)  /* refinement level this edge
 				     belongs to */
-#define MCE1EGLN   (MCE1EGLVL+1) /* length of one edge entry */
+#define MCE1EGNOD1 (MCE1EGLVL+1)  /* first node of the edge, there are
+				     2 nodes */
+#define MCE1EGLN   (MCE1EGNOD1+2) /* length of one P1 edge entry */
+/* DO NOT CHANGE THE ORDER after the EGLN entry !!! */
+#define MCE2EGNODM (MCE1EGLN)     /* midnode of the edge, only if 
+				     type >=31 and eg_w >=MCE2EGNODM */
 
 
 
@@ -764,5 +799,17 @@
 
 
 
+#define MCE2EHFAT1 0              /* father nodes (E2 => 10),
+				     numbering see doc/tetra_refine_elhier.svg
+				     (black and red nodes)
+				  */
+#define MCE2EHCHL1 (MCE2EHFAT1+10) /* child nodes (E2 => 25),
+				     numbering see doc/tetra_refine_elhier.svg
+				     (green nodes)
+				  */
+#define MCE2EHLVL  (MCE2EHCHL1+25) /* refinement level this entry
+				     belongs to */
+#define MCE2EHLN   (MCE2EHLVL +1) /* length of one hierarchy entry */
+
 
 #endif
