@@ -233,14 +233,18 @@ int navsto_adj_convert(struct navsto_matrix *K
     Cp_old.rows     = (*K).Cp.rows;
     Cp_old.cols     = (*K).Cp.cols;
     Cp_old.A        = (*K).Cp.A;
-    Cp_old.flex_rows= (*K).Cp.flex_rows;
+    Cp_old.flex_row_col_nr  = (*K).Cp.flex_row_col_nr;
+    Cp_old.flex_row_col_max = (*K).Cp.flex_row_col_max;
+    Cp_old.flex_row_cols    = (*K).Cp.flex_row_cols;
+    Cp_old.nr_processors    = (*K).Cp.nr_processors;
+    Cp_old.flex_reserve     = (*K).Cp.flex_reserve;
+    Cp_old.DD_proc_nr       = (*K).Cp.DD_proc_nr;
+    Cp_old.DD_waves_nr      = (*K).Cp.DD_waves_nr;
+    Cp_old.DD_nr            = (*K).Cp.DD_nr;
+    Cp_old.DD_rows_idx      = (*K).Cp.DD_rows_idx;
+    Cp_old.DD_rows          = (*K).Cp.DD_rows;
     
-    (*K).Cp.type     = SP_TYPE_UNDEF;
-    (*K).Cp.row_nr   = 0;
-    (*K).Cp.rows     = NULL;
-    (*K).Cp.cols     = NULL;
-    (*K).Cp.A        = NULL;
-    (*K).Cp.flex_rows= NULL;
+    sparse_init( &(*K).Cp );
     
     err=sparse_mat_transp( &Cp_old, &( (*K).Cp) );
     FUNCTION_FAILURE_HANDLE( err, sparse_mat_transp,
