@@ -3,7 +3,7 @@
     This file is part of
     FEINS, Finite Element Incompressible Navier-Stokes solver,
     which is expanding to a more general FEM solver and toolbox,
-    Copyright (C) 2003--2008, Rene Schneider 
+    Copyright (C) 2003--2013, Rene Schneider 
     <rene.schneider@mathematik.tu-chemnitz.de>
 
     This program is free software: you can redistribute it and/or modify
@@ -54,6 +54,8 @@ TO_HEADER:
 #include "feins_lapack.h"
 #include "sparse.h"
 #include "lin_solver.h"
+#include "linsolve_hypre_AMG.h"
+
 #include "mesh.h"
 #include "stokes_aux.h"
 #include "navstoassem.h"
@@ -382,6 +384,9 @@ int navsto_matrix_init(struct navsto_matrix *K, FIDX dim, FIDX lvlmax
   (*K).mg     = NULL;
   (*K).mg1    = NULL;
 
+  (*K).AMGdataFs= NULL;
+  (*K).AMGdataAp= NULL;
+
   (*K).mlsorters = NULL;
 
   for (i=0; i<5; i++)
@@ -1237,6 +1242,18 @@ int navsto_projector_w_precon(void *arg1, struct vector *in,
 		     navsto_advecdiff_tim_vec, navsto_projector_vel_W_cycle,
 		     K, &rhs, NULL); /* */
 	  break;
+	case 3:
+	  err=GMRES( MG_GMRES_RESTART, 500, 2, 0.0, (*K).innereps, 0,
+		     &xi, &resi, &iter,
+		     navsto_advecdiff_tim_vec, gen_hypre_AMG_preco,
+		     K, &rhs, (*K).AMGdataFs); /* */
+	  break;
+	case 4:
+	  err=gen_hypre_AMG_preco(NULL, &rhs, (*K).AMGdataFs, &xi);
+	  FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_preco,
+				   navsto_projector_w_precon);
+	  iter=2;
+	  break;
 	default:
 	  fprintf(stderr, "navsto_projector_w_precon: "
 		  "unknown velocity solver!\n");
@@ -1271,6 +1288,18 @@ int navsto_projector_w_precon(void *arg1, struct vector *in,
 		     navsto_advecdiff_tim_vec, navsto_projector_vel_W_cycle,
 		     K, &rhs, NULL); /* */
 	  break;
+	case 3:
+	  err=GMRES( MG_GMRES_RESTART, (*K).innersteps[0], 4, 0.0, 0.0, 0,
+		     &xi, &resi, &iter,
+		     navsto_advecdiff_tim_vec, gen_hypre_AMG_preco,
+		     K, &rhs, (*K).AMGdataFs); /* */
+	  break;
+	case 4:
+	  err=gen_hypre_AMG_preco(NULL, &rhs, (*K).AMGdataFs, &xi);
+	  FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_preco,
+				   navsto_projector_w_precon);
+	  iter=2;
+	  break;
 	default:
 	  fprintf(stderr, "navsto_projector_w_precon: "
 		  "unknown velocity solver!\n");
@@ -1356,6 +1385,8 @@ int navsto_projector_w_precon(void *arg1, struct vector *in,
   xi.len=pvx_nr;
   xi.n_max=pvx_nr;
   xi.V=&(*out).V[dim*vx_nr];
+
+#if ( FP_VELOCITY_SOLVER <4 )
   /* err=PCG( 10000, 2, 0.0, (*K).innereps, 0, &xi, &resi, &iter,
      sparse_mul_mat_vec, NULL, &(*K).Ap, &rhs, NULL); /* */
   if ((*K).innersteps[dim] <0)
@@ -1377,6 +1408,12 @@ int navsto_projector_w_precon(void *arg1, struct vector *in,
      &(*K).Ap, &rhs, K); /* */
   /* if (err=10) err=SUCCESS; /* */
   FUNCTION_FAILURE_HANDLE( err, PCG, navsto_projector_w_precon);
+#else
+  err=gen_hypre_AMG_preco(NULL, &rhs, (*K).AMGdataAp, &xi);
+  FUNCTION_FAILURE_HANDLE( err, gen_hypre_AMG_preco,
+			   navsto_projector_w_precon);
+  iter=2;
+#endif
   (*K).innercount[dim]+=iter;
 
   /* multiply by Fp:
@@ -1389,6 +1426,7 @@ int navsto_projector_w_precon(void *arg1, struct vector *in,
 			   navsto_projector_w_precon);
 
   /* solve Mp out = rhs */
+#ifndef STOKES_PRECO_CHEBYSHEV
   if ((*K).innersteps[dim+1] <0)
     {
       err=PCG( 10000, 2, 0.0, (*K).innereps, 0, &xi, &resi, &iter,
@@ -1404,6 +1442,16 @@ int navsto_projector_w_precon(void *arg1, struct vector *in,
       if (err==10) err=SUCCESS; /* */
     }
   FUNCTION_FAILURE_HANDLE( err, PCG, navsto_projector_w_precon);
+#else
+  iter=5;
+  resi=0;
+  err=chebyshev_semi_iteration(  
+	 &xi, iter, 
+	 sparse_mul_mat_vec, vector_diag_scale, 
+	 &(*K).M, &rhs,  &(*K).Minvdiag,
+	 0.5, 2.0);
+  FUNCTION_FAILURE_HANDLE( err, chebyshev_semi_iteration, navsto_projector_w_precon);
+#endif
   (*K).innercount[dim+1]+=iter;
 
   /* C^{-1} done, now P */
@@ -2477,11 +2525,11 @@ int navsto_precon_p_lapl_bpx(void *notused, struct vector *in,
      restricts the hierarchy entries which can be used, find the begin
      and end of those which can*/
   i=0;
-  while ((i<hi_nr)&&((*m).hier[i*hi_w+MCT2HILVL]<1)) i++;
+  while ((i<hi_nr)&&((*m).hier[i*hi_w+MCT2HILVL]<0)) i++;
   hi_start=i;
 
   i=hi_nr-1;
-  while ((i>=hi_start)&&((*m).hier[i*hi_w+MCT2HILVL]>lvlmax-1)) i--;
+  while ((i>=hi_start)&&((*m).hier[i*hi_w+MCT2HILVL]>lvlmax-2)) i--;
   hi_end=i;
 
 
@@ -2533,7 +2581,7 @@ int navsto_precon_p_lapl_bpx(void *notused, struct vector *in,
   level_old=lvlmax;
   for (i=hi_end; i>=hi_start; i--)
     {
-      level=(*m).hier[i*hi_w+MCT2HILVL];
+      level=(*m).hier[i*hi_w+MCT2HILVL]+1;
       if (level<=0) printf("error, level=%"dFIDX"\n",  level);
       if (level_old!=level)
 	{
@@ -2637,7 +2685,7 @@ int navsto_precon_p_lapl_bpx(void *notused, struct vector *in,
   level_old=0; 
   for (i=hi_start; i<=hi_end; i++)
     {
-      level=(*m).hier[i*hi_w+MCT2HILVL];
+      level=(*m).hier[i*hi_w+MCT2HILVL]+1;
 
       if (level_old!=level)
 	{
