@@ -516,17 +516,9 @@ int navsto_solver(struct mesh *msh1, FIDX lvl_max,
 		sol_p.V     = &sol.V[dim*vx_nr];
 		sol_p.len   = vx_nr;
 		sol_p.n_max = vx_nr;
-		err=mesh_write_solution_vtk_t2( &msh2, &sol_u, dim, &sol_p, 1,
+		err=mesh_write_solution_vtk_tx( &msh2, &sol_u, dim, &sol_p, 1,
 						20, "visual/2d_mesh_vtk_" );
-		FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t1,
-					 navsto_solver); /* */
-		err=mesh_write_solution_femplot_t2( &msh2, &sol_u, dim, 
-						20, "visual/navsto_vec__" );
-		FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_femplot_t2,
-					 navsto_solver); /* */
-		err=mesh_write_solution_femplot_t2( &msh2, &sol, dim+1, 
-						20, "visual/navsto_all__" );
-		FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_femplot_t2,
+		FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 					 navsto_solver); /* */
 	      }
 
