@@ -37,16 +37,32 @@ sparse.h
 #define __SPARSE_H__ 1
 
 
+#include "config.h"
+
 
 #include "feins_macros.h"
 
+
 #include "sparse_struct.h"
+#include "sparse_inline_functions.h"
 
 
-int sparse_flex_alloc(struct sparse *K, FIDX nrows
+int sparse_flex_alloc(struct sparse *K, FIDX nrows, FIDX cols_p_row
 /* creates a sparse matrix in the initial flexible format with nrows rows 
    
    Input:  nrows   - number of rows of the matrix,
+           cols_p_row
+                   - number of columns per row, this should be at
+                     least the average number of columns per row,
+                     if it is too small, it will cause large CPU
+                     overhead during assembly of the matrix, if to
+                     large, it causes memory waste
+		     the average number of columns per row under 
+		     uniform refinement:
+		     t1 -> 7
+		     t2 -> 12 
+		     e1 -> 15  -> vector valued  -> 3*15 ~ 45
+		     e2 -> 29  -> vector valued  -> 3*29 ~ 87 
 
    Output: K       - (K is given by reference), memory for the
                      neccessary fields in K is allocated,
@@ -56,16 +72,24 @@ int sparse_flex_alloc(struct sparse *K, FIDX nrows
            FAIL    - failure, see error message
 */);
 
-int sparse_alloc(struct sparse *K, FIDX nrows, FIDX notused_density
-/* OBSOLETE, kept for ease of switching to new matrix format;
-   creates a sparse matrix in the initial flexible format with nrows rows 
+int sparse_set_DD(struct sparse *K, FIDX DD_proc_nr, FIDX DD_waves_nr,
+		  FIDX *DD_node_idx, FIDX *DD_node, FIDX comps
+/* set the DD info of the sparse struct, for OMP parallelisation 
    
    Input:  nrows   - number of rows of the matrix,
            density - not used!, only present for backward compatibility
+	   DD_proc_nr
+	           - number of processors for which the DD was created
+	   DD_waves_nr
+	           - number of waves of DD_proc_nr independent domains
+	   DD_node_idx
+                   - DD_node_idx[i] index of start of subdomain i
+		     in DD_node (length DD_nr+1)
+	   DD_node - list of nodes in each subdomain
+	   comps   - number of block rows of the matrix for vector
+	             valued PDE problems, comps=1 for scalar PDES
 
-   Output: K       - (K is given by reference), memory for the
-                     neccessary fields in K is allocated,
-		     dimensions of K initialised, type set to flexible
+   In/Out: K       - (K is given by reference), the DD_* parts are set
 
    Return: SUCCESS - success,
            FAIL    - failure, see error message
@@ -114,11 +138,11 @@ int sparse_row_empty(struct sparse *K, FIDX row
            FAIL    - failure, the row doesn't belong to the matrix
 */);
 
-int sparse_mat_write_file(struct sparse *A, char *name
+int sparse_mat_write_file(struct sparse *K, char *name
 /* writes the contents of the sparse matrix to a file with the
    specified name
    
-   Input:  A       - sparse matrix, original
+   Input:  K       - sparse matrix, original
            name    - name for the file
 
    Output: (the file)
@@ -326,57 +350,6 @@ int sparse_mul_mat_vec_add_trans(struct sparse *K, struct vector *vec,
            FAIL    - failure, see error message
 */);
 
-int sparse_add_local(struct sparse *K, enum transposetype trans,
-		     FIDX m, FIDX row[], FIDX n,
-		     FIDX col[], double local[], FIDX LDlocal
-/* adds the matrix local to the by row and col defined submatrix of K,
-   if trans==NoTrans:
-   K(row[i],col[j])+=local[j*LDlocal+i];
-
-   if trans==Trans:
-   K(row[i],col[j])+=local[i*LDlocal+j];
-
-   Input:  trans   - ==NoTrans  => normal add
-                     ==Trans    => Transposed add
-           m       - number of rows in local, length of row
-           row     - array of row indices into K, length m
-           n       - number of collumns in local, length of col
-           col     - array of collumn indices into K, length n
-	   local   - array to be added to K, size m*n
-	   LDlocal - leading dimension of local
-
-   In/Out: K       - sparse matrix to which values are added
-
-   Return: SUCCESS - success,
-           FAIL    - failure, see error message
-*/);
-
-int sparse_get_entry(struct sparse *K, FIDX row, FIDX col,
-		     int create, double **entry
-/* if K contains an entry K(row,col) a pointer to this entry is
-   returned, such that 
-             (*entry)=K(row,col)
-   if it doesn't contain the entry yet and create==1, it is created
-   and a pointer to the entry is returned as before.
-
-   Input:   row    - row index into the sparse matrix K
-            col    - column index into the sparse matrix K
-	    create - if ==1 entry will be created (if possible)
-	             otherwise, no entry will be created, only
-	             existing ones are returned
-
-   In/Out:  K      - sparse matrix, eventually the entry K(row,col) is
-                     created
-
-   Output:  entry  - (by reference) pointer to a double such that 
-                     (*entry)=K(row,col)
-
-   Return: SUCCESS - success,
-           FAIL    - failure, entry not found and not allowed to
-                     create,
-		     if allowed, creation failed, see error message
-*/);
-
 int sparse_convert_compressed_row(struct sparse *K
 /* converts a matrix stored in flexible mode to compressed row mode
 
@@ -435,6 +408,23 @@ void vector_free(struct vector *V
                      released 
 */);
 
+int vector_random(struct vector *vec, int *seed
+/* fills the vector with (pseudo) random numbers of normal (0,1)
+   distribution as generated by LAPACK routine DLARNV
+      
+   In/Out: seed    - array of seed values, length 4,
+                     from documentation of DLARNV:
+                     on entry, the seed of the random number generator;
+		     the array elements must be between 0 and 4095,
+		     and seed[3] must be odd;
+		     on exit, the seed is updated.
+
+   Output: vec     - vector, the contents are assigned 
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
 int vector_diag_scale(void *notused, struct vector *in,
 		      void *arg3, struct vector *out
 /* performs
@@ -541,6 +531,55 @@ void projector1_free(struct projector1 *V
                      released 
 */);
 
+int projectorMorthoZ_no_precon(void *notused, struct vector *in,
+			 void *arg3, struct vector *out
+/* performs
+     out = P*in
+   where P is a M-orthogonal projector, such that the result is
+   M-orthogonal to all vectors in Z,
+   it is safe to have in==out
+
+   Input:  notused - well, it is not used but included in the
+                     interface to allow this function to be used as a
+                     "preconditioner" 
+           in      - input vector
+	   arg3=
+	   P       - projector data
+
+   Output: out    - (given by reference), the projection of in
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
+int projectorMorthoZ_init(struct projectorMorthoZ *P, FIDX num,
+			  struct vector **Z, struct sparse *M
+/* allocates memory in V 
+   
+   Input:  num     - number of vectors Z
+           M       - Mass matrix, a reference is stored in P,
+                     P should be destroyed before M
+
+   In/Out: Z       - array of num vectors to which P shall 
+                     M-ortho-project,
+		     the vectors are ortho-normalised, then the
+		     pointer to them is stored (without copy),
+		     (Z will be freed with P)
+
+   Output: P       - (given by reference), all data is initialised
+
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
+void projectorMorthoZ_free(struct projectorMorthoZ *P
+/* memory allocated in P is freed, so that P can be freed
+
+   Output: P       - (P is given by reference) inside memory is
+                     released 
+*/);
+
 void int_double_list_free( struct int_double_list *root
 /* free all enties of a list given by the root entry
    
@@ -591,6 +630,66 @@ int coarse_mat_set( struct sparse *K, FIDX nr_diri, FIDX *diris,
            FAIL    - failure, see error message
 */);
 
+int sparse_solve_HypreAMG_cmat_set( struct sparse *K, 
+	FIDX nr_diri, FIDX *diris,
+	int symm, struct coarse_mat *cmat
+/* initialises hypreAMG (Algebraic Multi Grid) for the stiffness matrix K
+   and stores its data in cmat, such that this is suitable for the coarse
+   grid solver, HypreAMG versions of coarse_mat_set
+   
+   Input:  K       - the stiffness matrix for the coarse grid
+           nr_diri - number of degrees of freedom (DOFs) which are
+	             subject to Dirichlet boundary conditions, such
+	             that the according rows and columns of the
+	             matrix have to be deleted 
+           diris   - vector of length nr_diri, stating which DOFs
+                     these are
+	   symm    - marking the symmetry properties of the matrix,
+	             if the matrix is known to be symmetric positive
+	             definit, use symm=1 (so more efficient methods
+	             can be used), 
+		     if the matrix is known to have symmetric
+		     connectivity but the matrix is not necessarily
+		     symmetric, use symm=2 
+
+   Output: cmat    - (given by reference) on output it will contain
+                     the data for the coarse grid solver, internal
+                     fields are initialised and memory allocated
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
+int sparse_solve_UMFPACK_cmat_set( struct sparse *K, 
+	FIDX nr_diri, FIDX *diris,
+	int symm, struct coarse_mat *cmat
+/* computes the triangular factorisation of the stiffness matrix K
+   and stores it in cmat, such that this is suitable for the coarse
+   grid solver, UMFPACK versions of coarse_mat_set
+   
+   Input:  K       - the stiffness matrix for the coarse grid
+           nr_diri - number of degrees of freedom (DOFs) which are
+	             subject to Dirichlet boundary conditions, such
+	             that the according rows and columns of the
+	             matrix have to be deleted 
+           diris   - vector of length nr_diri, stating which DOFs
+                     these are
+	   symm    - marking the symmetry properties of the matrix,
+	             if the matrix is known to be symmetric positive
+	             definit, use symm=1 (so more efficient methods
+	             can be used), 
+		     if the matrix is known to have symmetric
+		     connectivity but the matrix is not necessarily
+		     symmetric, use symm=2 
+
+   Output: cmat    - (given by reference) on output it will contain
+                     the data for the coarse grid solver, internal
+                     fields are initialised and memory allocated
+
+   Return: SUCCESS - success,
+           FAIL    - failure, see error message
+*/);
+
 int coarse_mat_solve( struct coarse_mat *cmat, enum transposetype trans,
 		      struct vector *rhs, struct vector *x
 /* solves the equation system 
