@@ -34,6 +34,9 @@
 #ifndef SPARSE_STRUCT_H
 #define SPARSE_STRUCT_H
 
+// config (by cmake)
+#include "config.h"
+#include "linsolve_struct.h" /* for UMFPACK coarse_mat, if available */
 
 /* sparse matrix data structure */
 /* 23.03.2007 rework to compressed row structure,
@@ -49,6 +52,45 @@
 #define SP_TYPE_UNDEF -1 
 #define SP_TYPE_FLEX 1 
 #define SP_TYPE_COMPROW 2 
+
+/* minimum (assymptotic) values, actual minimum (no new violations) value:
+   #define FEINS_SPARSE_COLS_P_ROW_T1 7,  7
+   #define FEINS_SPARSE_COLS_P_ROW_T2 12, 19
+   #define FEINS_SPARSE_COLS_P_ROW_E1 15, 20--23
+   #define FEINS_SPARSE_COLS_P_ROW_E2 29, 65--105 */
+/* values used: */
+#define FEINS_SPARSE_COLS_P_ROW_T1 8
+#define FEINS_SPARSE_COLS_P_ROW_T2 20
+#define FEINS_SPARSE_COLS_P_ROW_E1 21
+#define FEINS_SPARSE_COLS_P_ROW_E2 70
+
+struct sparse_flex_col_block {
+  FIDX   *cols_idx;
+  double *cols_data;
+                      /* arrays which store the column indices and
+			 the data (matrix entry) for this column, 
+			 the arrays are of size 
+			 sparse.flex_row_col_max, known to the parent
+			 of the blocks, the parent also knows how many
+			 entries in total are valid:
+			 sparse.flex_row_col_nr[i]
+		      */ 
+  struct sparse_flex_col_block *next;
+                      /* next block, if sparse.flex_row_col_max did
+			 not suffice for sparse.flex_row_col_nr[i], 
+			 NULL as long as it is not used */
+};
+
+struct sparse_flex_reserve_list {
+  FIDX  nr;           /* number of entries in use */
+  FIDX  max;          /* number of entries allocated */
+  struct sparse_flex_col_block *blocks;
+                      /* the reserve blocks of this list entry */
+  struct sparse_flex_reserve_list *next;
+                      /* the next entry of the reserve list,
+			 NULL if none */
+};
+
 struct sparse {
   int  type;          /* describes the actual storage type of the
 			 current matrix:
@@ -79,12 +121,43 @@ struct sparse {
 
 
   /* flexible storage stage, only used when type==1 */
-  struct int_double_list *flex_rows;
-                      /* array containing the roots of the simply
-			 connected lists for each row,
-		         each row will be sorted as it is created,
-		         with the root element being the dioagonal
-		         entry */
+  FIDX  *flex_row_col_nr;
+                      /* array storing the number of columns currently 
+			 in each row */
+  FIDX  flex_row_col_max;
+                      /* maximum number of entries (memory allocated)
+			 of each colums block */
+  struct sparse_flex_col_block *flex_row_cols;
+                      /* array of one column block for each row,
+			 which store the column indices and
+			 the data (matrix entry) for this row+column, 
+			 the diagonal entry is always first
+			 flex_row_cols[i] is a bock of size 
+			 flex_row_col_max, of which the first 
+			 flex_row_col_nr[i] entries are valid. If
+			 these are more than flex_row_col_max, then
+			 flex_row_cols.next points to the next block,
+			 which may point to the block again and so on.
+		      */
+  int nr_processors;  /* number of processors, */
+  struct sparse_flex_reserve_list *flex_reserve;
+                      /* each processor has its own reserve blocks,
+			 which it appends tp flex_row_cols[i].next if
+			 necessary, these reserve blocks are stored in
+			 a list, the list is extended if they don't
+			 suffice */
+
+  FIDX DD_proc_nr;    /* number of processors to be used (DD) */
+  FIDX DD_waves_nr;   /* number of waves of DD_proc_nr domains (DD) */
+  FIDX DD_nr;         /* number of subdomains in Domain Decomposition
+			 (DD), should be DD_proc_nr*DD_waves_nr */
+  FIDX *DD_rows_idx;  /* DD_rows_idx[i] index of start of subdomain i
+			 in DD_rows (length DD_nr+1) */
+  FIDX *DD_rows;      /* list of rows in each subdomain,
+			 DD_rows[DD_rows_idx[i]]
+			 ... DD_rows[DD_rows_idx[i+1]-1] */
+
+  
 #ifdef HAVE_OPENMP
   /* coloring of dofs in matrix */
   FIDX n_col_buckets; /* number of colors in the coloring of dofs */
@@ -134,6 +207,16 @@ struct projector1 {
 		      */
 };
 
+/* M-orthogonal to Z projector data structure */
+struct projectorMorthoZ {
+  FIDX num;           /* number of vectors Z */
+  struct vector **Z;  /* pointer to array num vectors, such that 
+			 Z[i] is a vector to which the result shall be
+			 M-orthogonal
+		      */
+  struct sparse *M;   /* corresponding Mass matrix */
+  struct vector My;   /* vector for storing M*y */
+};
 
 /* general transpose / notranspose stuff */
 enum transposetype {
@@ -173,6 +256,24 @@ struct coarse_mat
   int *ipiv;          /* integer vector of length dofs, to store the
 			 pivot indices, only used if symm=2, otherwise
 			 the pointer will be ==NULL */
+
+  /* stuff related to HypreAMG variant of coarse_mat: */
+  int HYPREamgIsInit;
+#ifdef HAVE_HYPRE_NO_MPI
+  void *AMGdata;
+#endif
+
+
+  /* stuff related to UMFPACK variant of coarse_mat: */
+  int UMFPACKisinit;
+#ifdef HAVE_UMFPACK
+  struct linsolver UMFPACKdata;
+  struct projector1 P;
+  FIDX *UMFPACKisdiri;
+  struct vector UMFPACKhelp;
+  /* rhs is not modified by solver, if input x=0 */
+#endif
+
 };
 
 
