@@ -1146,8 +1146,8 @@ int stokes_write_streamfunc_t2( struct mesh *m, struct vector *usol,
       return FAIL;
     }
 
-  err=sparse_alloc( &K, (*m).vx_nr, 32);
-  FUNCTION_FAILURE_HANDLE( err, sparse_alloc, stokes_write_streamfunc_t21);
+  err=sparse_flex_alloc( &K, (*m).vx_nr, FEINS_SPARSE_COLS_P_ROW_T2);
+  FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_write_streamfunc_t21);
   err=vector_alloc( &rhs, (*m).vx_nr );
   FUNCTION_FAILURE_HANDLE( err, vector_alloc, stokes_write_streamfunc_t21);
   err=vector_alloc( &strf, (*m).vx_nr );
