@@ -102,9 +102,79 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
                      valid
 */ 
 		     ){
+  FIDX *DD_dofs, *DD_dofs_idx, i, j, dim=2;
+  int err;
+  FIDX pvx_nr;           /* number of pressure dofs */
+  FIDX vx_nr;
+  struct sparse *Fs;     /* pointer to the stiffness matrices for this
+			    level */
+
+  vx_nr = (*m).vx_nr;
+
+  (*K).dim    = dim;
+  if (dim > 2)
+    {
+      /* cry */
+      fprintf(stderr,
+	      "stokes_assem_t21: dim >2 not implemented (Jacinv)\n");
+      return FAIL;
+    }
+
+  /* reinitialise the parts of the matrix struct which will be
+     redefined */
+  err=navsto_matrix_reinit( K, vx_nr );
+  FUNCTION_FAILURE_HANDLE( err, navsto_matrix_reinit, stokes_assem_t21);
+  (*K).msh=m;
+  
+  TRY_MALLOC( (*K).ml1, 1, struct multilvl, stokes_assem_t21);
+  err=multilvl_init_t2( m, 1, (*K).ml1 );
+  FUNCTION_FAILURE_HANDLE( err, multilvl_init_t2, stokes_assem_t21);
+  
+  TRY_MALLOC( (*K).bpx, 1, struct bpxdata, stokes_assem_t21);
+  err=bpx_init_tx( (*K).bpx, (*K).ml1 );
+  FUNCTION_FAILURE_HANDLE( err, bpx_init_tx, stokes_assem_t21);
+  
+  /* initialise K */
+  (*K).vx_nr  = vx_nr;
+  
+  err=stokes_pdof_init_t21( m, K);
+  FUNCTION_FAILURE_HANDLE( err, stokes_pdof_init_t21, stokes_assem_t21);
+  pvx_nr=(*K).pvx_nr;
+  
+  /* allocate space for Fs if needed */
+  if (noFs!=1)
+    {
+      Fs=&(*K).Fs[lvl];
+      err=sparse_flex_alloc( Fs, vx_nr, dim*FEINS_SPARSE_COLS_P_ROW_T2);
+      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
+      
+      TRY_MALLOC( (*K).mg, 1, struct mgdata, stokes_assem_t21);
+    }
+  else Fs=NULL;
+  
+  for (i=0; i<dim; i++)
+    {
+      err=sparse_flex_alloc( &(*K).Bs[i], pvx_nr, FEINS_SPARSE_COLS_P_ROW_T2);
+      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
+    }
+  err=sparse_flex_alloc( &(*K).M, pvx_nr, FEINS_SPARSE_COLS_P_ROW_T1);
+  FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
+
+
+  /* initialise rhs, u0 */
+  err=vector_alloc( rhs, dim*vx_nr+pvx_nr );
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, stokes_assem_t21);
+  err=vector_alloc( u0,  dim*vx_nr+pvx_nr );
+  FUNCTION_FAILURE_HANDLE( err, vector_alloc, stokes_assem_t21);
+
+#pragma omp parallel
+  {
   int  err;
   FIDX el, i, j, k, r;
 
+  FIDX iel;
+  int my_id, OMP_num_threads, DD_num_threads, wave;
+
   FIDX dim=2, bas_n1, bas_n2, vx_nr, eg_nr, hi_nr, 
     el_w, vx_w, eg_w, fc_w, bd_w, hi_w, fu_w,
     bigN;
@@ -115,10 +185,6 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
   double dhelp, done=1.0, dzero=0.0;
   int fone=1, fdim, fbas_n1, fbas_n2; 
 
-
-  struct sparse *Fs;     /* pointer to the stiffness matrices for this
-			    level */
-
   struct int_data iform; /* integration formula 2d   */
   double *elF;           /* element viscous stiffness matrix */
   double *elB;           /* element pressure stiffness matrix */
@@ -141,7 +207,6 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
 			    lengths in the mesh are given),
 			    nu is taken from m.param[MC2XPANUPO] */
 
-  FIDX pvx_nr;           /* number of pressure dofs */
 
   double *phi1, *phi2, *gradp1, *gradp2;
 
@@ -180,62 +245,6 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
 
   nu    = (*m).para[MC2XPANUPO];
 
-  if (dim > 2)
-    {
-      /* cry */
-      fprintf(stderr,
-	      "stokes_assem_t21: dim >2 not implemented (Jacinv)\n");
-      return FAIL;
-    }
-
-  /* reinitialise the parts of the matrix struct which will be
-     redefined */
-  err=navsto_matrix_reinit( K, vx_nr );
-  FUNCTION_FAILURE_HANDLE( err, navsto_matrix_reinit, stokes_assem_t21);
-  (*K).msh=m;
-
-  TRY_MALLOC( (*K).ml1, 1, struct multilvl, stokes_assem_t21);
-  err=multilvl_init_t2( m, 1, (*K).ml1 );
-  FUNCTION_FAILURE_HANDLE( err, multilvl_init_t2, stokes_assem_t21);
-
-  TRY_MALLOC( (*K).bpx, 1, struct bpxdata, stokes_assem_t21);
-  err=bpx_init_tx( (*K).bpx, (*K).ml1 );
-  FUNCTION_FAILURE_HANDLE( err, bpx_init_tx, stokes_assem_t21);
-
-  /* initialise K */
-  (*K).vx_nr  = vx_nr;
-  (*K).dim    = dim;
-
-  err=stokes_pdof_init_t21( m, K);
-  FUNCTION_FAILURE_HANDLE( err, stokes_pdof_init_t21, stokes_assem_t21);
-  pvx_nr=(*K).pvx_nr;
-
-  /* allocate space for Fs if needed */
-  if (noFs!=1)
-    {
-      Fs=&(*K).Fs[lvl];
-      err=sparse_flex_alloc( Fs, vx_nr);
-      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
-
-      TRY_MALLOC( (*K).mg, 1, struct mgdata, stokes_assem_t21);
-    }
-  else Fs=NULL;
-
-  for (i=0; i<dim; i++)
-    {
-      err=sparse_flex_alloc( &(*K).Bs[i], pvx_nr);
-      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
-    }
-  err=sparse_flex_alloc( &(*K).M, pvx_nr);
-  FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, stokes_assem_t21);
-
-
-  /* initialise rhs, u0 */
-  err=vector_alloc( rhs, dim*vx_nr+pvx_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, stokes_assem_t21);
-  err=vector_alloc( u0,  dim*vx_nr+pvx_nr );
-  FUNCTION_FAILURE_HANDLE( err, vector_alloc, stokes_assem_t21);
-
   /* allocate memory for elF */
   TRY_MALLOC( elF, bas_n2*bas_n2, double, stokes_assem_t21);
   /* allocate memory for elB */
@@ -253,16 +262,49 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
   TRY_MALLOC( dofs2, bas_n2, FIDX, stokes_assem_t21);
   bigN = dim*vx_nr+pvx_nr;
 
-  /* clear rhs, u0 */
-  for (i=0; i<bigN; i++)
-    {
-      (*rhs).V[i]=0.0;
-      (*u0).V[i] =0.0;
-    }
 
-  /* loop over all elements */
-  for (el=0; el<(*m).el_nr; el++)
+#pragma omp master
+  {
+    /* clear rhs, u0 */
+    for (i=0; i<bigN; i++)
+      {
+	(*rhs).V[i]=0.0;
+	(*u0).V[i] =0.0;
+      }
+  }
+
+
+#ifdef HAVE_OPENMP
+  my_id      = omp_get_thread_num();
+  OMP_num_threads = omp_get_num_threads();
+#else
+  my_id      = 0;
+  OMP_num_threads = 1;
+#endif
+
+  if (OMP_num_threads<(*m).DD_proc_nr)
     {
+      fprintf(stderr,
+	      "assem_poisson_t1_t2: wrong number of treads\n"
+	      "   is %d < m.DD_proc_nr=%"dFIDX"\n",
+	      OMP_num_threads, (*m).DD_proc_nr);
+      FEINS_FAILURE_ACTION;
+    }
+  DD_num_threads=(*m).DD_proc_nr;
+  /* wait till all threads have finished init phase */
+#pragma omp barrier
+
+  /* loop over all elements, in parallel */
+  for (wave=0; wave<(*m).DD_waves_nr; wave++)
+    {
+      if (my_id<DD_num_threads)
+	{
+       for (iel=(*m).DD_elem_idx[wave*DD_num_threads+my_id]; 
+            iel<(*m).DD_elem_idx[wave*DD_num_threads+my_id+1];
+            iel++)
+	 {
+      el=(*m).DD_elem[iel];
+
       /* set elF to zero */
       for (i=0; i<bas_n2*bas_n2; i++) 
 	{
@@ -393,8 +435,11 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
 			   bas_n1, dofs1, bas_n1, dofs1,
 			   elM, bas_n1 );
       FUNCTION_FAILURE_HANDLE( err, sparse_add_local, stokes_assem_t21);
-
-    } /* end loop over all elements */
+	 } /* for iel */
+	} /*  if (my_id<DD_num_threads) */
+#pragma omp barrier       
+    } /* for wave */
+  /* end loop over all elements */
 
 #ifdef FEINS_have_warning 
 #warning "stokes_T21: Neumann BC not accounted for in rhs yet!"
@@ -404,60 +449,14 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
 
   /* K itself is ready, now implement boundary conditions  */
 
-  err=stokes_diri_bc_init_t2( m, u0, rhs, K);
-  FUNCTION_FAILURE_HANDLE( err, stokes_diri_bc_init_t2, stokes_assem_t21);
+#pragma omp master
+  {
+    err=stokes_diri_bc_init_t2( m, u0, rhs, K);
+    FUNCTION_FAILURE_HANDLE( err, stokes_diri_bc_init_t2, stokes_assem_t21);
 
-  /* set the level info in K */
-  (*K).lvl=lvl;
-
-  if (noFs!=1)
-    {
-      if ( (*Fs).type == SP_TYPE_FLEX )
-	{
-	  err = sparse_convert_compressed_row( Fs );
-	  FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
-				   stokes_assem_t21 );
-	}
-
-      /* initialise the multigrid stuff */
-      err=mg_init_tx( (*K).Fs, m, (*K).ml1, (*K).mg, NULL );
-      FUNCTION_FAILURE_HANDLE( err, mg_init_tx, stokes_assem_t21);
-
-      for (i=0; i<(*K).bn_nr; i++)
-	for (j=0; j<=(*(*K).ml1).lmax; j++)
-	  {
-	    FIDX node;
-	    node=(*K).nodes[i];
-	    MLVLFINDENTRY(node, node, j, *(*K).ml1);
-	    if (node>=0)
-	      {
-		(*(*K).mg).invdiag[node]=0.0;
-	      }
-	  }
-    }
-
-
-  for (i=0; i<dim; i++)
-    {
-      if ( (*K).Bs[i].type == SP_TYPE_FLEX )
-	{
-	  err = sparse_convert_compressed_row( &(*K).Bs[i] );
-	  FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
-				   stokes_assem_t21);
-	}
-    }
-  if ( (*K).M.type == SP_TYPE_FLEX )
-    {
-      err = sparse_convert_compressed_row( &(*K).M );
-      FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
-			       stokes_assem_t21 );
-    }
-
-  /* store the inverse of the diagonal of M in Minvdiag, to use it in
-     a Jacobi preconditioner */
-  /* printf("vx_nr=%d    pvx_nr=%d\n", (int) vx_nr, (int) pvx_nr); /* */
-  err=sparse_extract_invdiag( &K->M, &K->Minvdiag);
-  FUNCTION_FAILURE_HANDLE( err, sparse_extract_invdiag, stokes_assem_t21);
+    /* set the level info in K */
+    (*K).lvl=lvl;
+  } /* end OMP master */
 
   /* free local data */
   free_intdata (&iform);
@@ -469,6 +468,89 @@ int stokes_assem_t21(struct navsto_matrix *K, struct vector *rhs,
   free(Jinvgrad);
   free(dofs1);
   free(dofs2);
+  } /* end OMP parallel */
+
+
+  /* generate a tailored DD_node for the dofs */
+  TRY_MALLOC( DD_dofs, pvx_nr, FIDX, stokes_assem_t21 );
+  TRY_MALLOC( DD_dofs_idx, (*m).DD_nr+1, FIDX, stokes_assem_t21 );
+
+  DD_dofs_idx[0]=0;
+  for (i=0; i<(*m).DD_nr; i++)
+    {
+      DD_dofs_idx[i+1]=DD_dofs_idx[i];
+      for (j=(*m).DD_node_idx[i];j<(*m).DD_node_idx[i+1]; j++) 
+	{
+          FIDX tdof=(*m).DD_node[j];
+	  if (tdof<pvx_nr)
+	    {
+              /* add this to DD_dofs */
+              DD_dofs[DD_dofs_idx[i+1]]=tdof;
+	      DD_dofs_idx[i+1]++;
+            }
+        }
+    }
+
+  if (noFs!=1)
+    {
+      if ( (*Fs).type == SP_TYPE_FLEX )
+	{
+	  err = sparse_convert_compressed_row( Fs );
+	  FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
+                                   stokes_assem_t21 );
+
+	  err = sparse_set_DD( Fs, (*m).DD_proc_nr,(*m).DD_waves_nr,
+                               (*m).DD_node_idx, (*m).DD_node, dim);
+	  FUNCTION_FAILURE_HANDLE( err, sparse_set_DD, stokes_assem_t21 );
+        }
+
+      /* initialise the multigrid stuff */
+      err=mg_init_tx( (*K).Fs, m, (*K).ml1, (*K).mg, NULL );
+      FUNCTION_FAILURE_HANDLE( err, mg_init_tx, stokes_assem_t21);
+
+      for (i=0; i<(*K).bn_nr; i++)
+	for (j=0; j<=(*(*K).ml1).lmax; j++)
+	  {
+            FIDX node;
+	    node=(*K).nodes[i];
+	    MLVLFINDENTRY(node, node, j, *(*K).ml1);
+	    if (node>=0)
+	      {
+                (*(*K).mg).invdiag[node]=0.0;
+              }
+	  }
+    }
+  for (i=0; i<dim; i++)
+    {
+      if ( (*K).Bs[i].type == SP_TYPE_FLEX )
+	{
+          err = sparse_convert_compressed_row( &(*K).Bs[i] );
+	  FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
+                                   stokes_assem_t21);
+
+	  err = sparse_set_DD( &(*K).Bs[i], (*m).DD_proc_nr,(*m).DD_waves_nr,
+                               DD_dofs_idx, DD_dofs, 1);
+	  FUNCTION_FAILURE_HANDLE( err, sparse_set_DD, stokes_assem_t21 )
+        }
+    }
+  if ( (*K).M.type == SP_TYPE_FLEX )
+    {
+      err = sparse_convert_compressed_row( &(*K).M );
+      FUNCTION_FAILURE_HANDLE( err, sparse_convert_compressed_row, 
+                               stokes_assem_t21 );
+      err = sparse_set_DD( &(*K).M, (*m).DD_proc_nr,(*m).DD_waves_nr,
+                           DD_dofs_idx, DD_dofs, 1);
+      FUNCTION_FAILURE_HANDLE( err, sparse_set_DD, stokes_assem_t21 )
+    }
+
+  /* store the inverse of the diagonal of M in Minvdiag, to use it in
+     a Jacobi preconditioner */
+  /* printf("vx_nr=%d    pvx_nr=%d\n", (int) vx_nr, (int) pvx_nr); /* */
+  err=sparse_extract_invdiag( &K->M, &K->Minvdiag);
+  FUNCTION_FAILURE_HANDLE( err, sparse_extract_invdiag, stokes_assem_t21);
+  
+  free(DD_dofs_idx);
+  free(DD_dofs);
 
   return SUCCESS;
 }
