@@ -74,6 +74,20 @@ int main(int argc, char *argv[])
   double t0, ti, t0l;
 
 
+#ifdef HAVE_OPENMP
+#pragma omp parallel
+  {
+#pragma omp master
+    {
+      printf("OpenMP num_threads=%d\n",omp_get_num_threads());
+      printf("  should be number of phyisical cores, use environment variable "
+	     " OMP_NUM_THREADS to modify,\n");
+      printf("  e.g. (bash shell): \"export OMP_NUM_THREADS=4\" \n");
+      printf("  e.g. (most other): \"set OMP_NUM_THREADS 4\" \n");
+    }
+  }
+#endif
+
   if (argc>1)
     {
       printf("meshfile: %s \n", argv[1]);
@@ -135,11 +149,11 @@ int main(int argc, char *argv[])
       err=vector_alloc( &rhs, vx_nr );
       FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
 
-      err=sparse_flex_alloc( &Ks[lvl], vx_nr );
+      err=sparse_flex_alloc( &Ks[lvl], vx_nr, FEINS_SPARSE_COLS_P_ROW_T2 );
       FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, main);
 
       err=projector1_alloc( &P, vx_nr );
-      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, main);
+      FUNCTION_FAILURE_HANDLE( err, projector1_alloc, main);
       
       /* init multilvl data */
       err=multilvl_init_tx( &msh2, 1, &ml, 2);
@@ -188,6 +202,7 @@ int main(int argc, char *argv[])
       err=assem_poisson_t2( &Ks[lvl], &rhs, x, &P, &msh2 );
       FUNCTION_FAILURE_HANDLE( err, assem_poisson_t2, main);
 
+      fprintf(stderr,"DEBUG: done assem\n");
 
       /* define the multigrid data */
       err=mg_init_tx( Ks, &msh2, &ml, &mg, &P);
@@ -243,12 +258,14 @@ int main(int argc, char *argv[])
 	 projector1_no_precon, &Ks[lvl], &rhs, &P );
 	 FUNCTION_FAILURE_HANDLE( err, PCG, main); /* */
 
-      /* strcpy(solver,"PCG_MG");
+      fprintf(stderr,"DEBUG: starting solver\n");
+      
+      strcpy(solver,"PCG_MG");
       err=PCG( 10000, 2, atol, rtol, 1, x, &resi, &iter, sparse_mul_mat_vec, 
 	       gen_proj_MG_tx, &Ks[lvl], &rhs, &mg );
       FUNCTION_FAILURE_HANDLE( err, PCG, main); /* */
       
-      strcpy(solver,"PCG_AMG");
+      /* strcpy(solver,"PCG_AMG");
 	{
 	void *AMGdata;
 	err=gen_hypre_AMG_setup(&Ks[lvl], P.len, P.V, 1, &AMGdata, 1 );
@@ -275,9 +292,9 @@ int main(int argc, char *argv[])
 
       if (lvl==lmax)
 	{
-	  err= mesh_write_solution_vtk_t2(&msh2, NULL, 0, x, 1,
+	  /*err= mesh_write_solution_vtk_tx(&msh2, NULL, 0, x, 1,
 					  15,"visual/poisson" );
-	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2, main);/* */
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx, main);/* */
 	}
 
       mg.cmat=NULL; /* prevent it from beeing destroyed, so we don't
