@@ -301,9 +301,9 @@ int main(int argc, char *argv[])
 
 
   /* allocate memory for matrix K and the vectors */
-  err=sparse_flex_alloc( &A, vx_nr);
-  err=sparse_flex_alloc( &M, vx_nr);
-  err=sparse_flex_alloc( &K, vx_nr);
+  err=sparse_flex_alloc( &A, vx_nr, FEINS_SPARSE_COLS_P_ROW_T2);
+  err=sparse_flex_alloc( &M, vx_nr, FEINS_SPARSE_COLS_P_ROW_T2);
+  err=sparse_flex_alloc( &K, vx_nr, FEINS_SPARSE_COLS_P_ROW_T2);
   err=projector1_alloc( &P, vx_nr );
 
   x=&xtrial1;
@@ -437,17 +437,6 @@ int main(int argc, char *argv[])
 	    
 	    TIMEGET;
 	    ts=ti-t0;
-
-
-	    if (task==1)
-	      {
-		/* recompute the system matrices, as they were
-		   changed by the UMFPACK factorization phase */
-		err=assem_conv_diff_tx( NULL, NULL,  &b, NULL, &maxPe, eps,
-					&A, NULL, 1.0, 1.0/(tau*gamma), 0.0, 1.0/tau,
-					&msh2, 1, time, 2); /* */
-		FUNCTION_FAILURE_HANDLE( err, assem_conv_diff_tx, main);
-	      }
 	  }
 
 
@@ -693,9 +682,9 @@ int main(int argc, char *argv[])
 	  
 
 	  /* write the solution for paraview visualisation */
-	  err=mesh_write_solution_vtk_t2( &msh2, NULL, 0, &xwrite, 1,
+	  err=mesh_write_solution_vtk_tx( &msh2, NULL, 0, &xwrite, 1,
 					  strlen(fname)+1,fname );
-	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t1,
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 				   main); /* */
 
 	  t_write += tau_write;
