@@ -106,6 +106,21 @@ int main(int argc, char *argv[])
   double resi=0.0;
 
   char *buffer;
+
+
+#ifdef HAVE_OPENMP
+#pragma omp parallel
+  {
+#pragma omp master
+    {
+      printf("OpenMP num_threads=%d\n",omp_get_num_threads());
+      printf("  should be number of phyisical cores, use environment variable "
+	     " OMP_NUM_THREADS to modify,\n");
+      printf("  e.g. (bash shell): \"export OMP_NUM_THREADS=4\" \n");
+      printf("  e.g. (most other): \"set OMP_NUM_THREADS 4\" \n");
+    }
+  }
+#endif
   
   
   if (argc>1)
@@ -176,8 +191,8 @@ int main(int argc, char *argv[])
 
 
   /* allocate memory for matrix K and the vectors */
-  err=sparse_flex_alloc( &A, vx_nr);
-  err=sparse_flex_alloc( &B, vx_nr);
+  err=sparse_flex_alloc( &A, vx_nr, FEINS_SPARSE_COLS_P_ROW_T2);
+  err=sparse_flex_alloc( &B, vx_nr, FEINS_SPARSE_COLS_P_ROW_T2);
   err=projector1_alloc( &P, vx_nr );
   err=vector_alloc( &x, vx_nr );
   err=vector_alloc( &rhs, vx_nr );
@@ -339,15 +354,6 @@ int main(int argc, char *argv[])
 		TIMEGET;
 		printf("tstep=%5"dFIDX"  t=%8.6f   solver UMFPACK, size=%6"dFIDX", t_assem=%10.3e, t_mul=%10.3e, t_solver=%10.3e\n", tstep, t, rhs.len, ta, tb, ti-t0);
 
-		if (task==1)
-		  {
-		    /* recompute the system matrices, as they were
-		       changed by the UMFPACK factorization phase */
-		    err=assem_conv_diff_tx( NULL, NULL,  &b, NULL, &maxPe, eps,
-					    &A, &B, 1.0, 1.0/tau, 0.0, 1.0/tau,
-					    &msh2, 1, t, 2); /* */
-		    FUNCTION_FAILURE_HANDLE( err, assem_conv_diff_tx, main);
-		  }
 	      }
 	      break;
 	    default:
@@ -364,9 +370,9 @@ int main(int argc, char *argv[])
 	  sprintf(fname, "visual/conv_diff_vtk_%05"dFIDX,
 		  (tstep/write_steps));
 	  /* write the solution for paraview visualisation */
-	  err=mesh_write_solution_vtk_t2( &msh2, NULL, 0, &x, 1,
+	  err=mesh_write_solution_vtk_tx( &msh2, NULL, 0, &x, 1,
 					  strlen(fname)+1,fname );
-	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t1,
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 				   main); /* */
 	}
 
