@@ -88,6 +88,21 @@ int main(int argc, char *argv[])
   stiffening = 1;  /* ==1 for real Lame', 
                       ==0 for stiffening of small elements */
 
+
+#ifdef HAVE_OPENMP
+#pragma omp parallel
+  {
+#pragma omp master
+    {
+      printf("OpenMP num_threads=%d\n",omp_get_num_threads());
+      printf("  should be number of phyisical cores, use environment variable "
+	     " OMP_NUM_THREADS to modify,\n");
+      printf("  e.g. (bash shell): \"export OMP_NUM_THREADS=4\" \n");
+      printf("  e.g. (most other): \"set OMP_NUM_THREADS 4\" \n");
+    }
+  }
+#endif
+
   if (argc>1)
     {
       printf("meshfile: %s \n", argv[1]);
@@ -178,8 +193,8 @@ int main(int argc, char *argv[])
 		 " (%"dFIDX"+1!= %"dFIDX")\n", msh2.lvl, lvl);
 	  return FAIL;
         }
-      err=sparse_flex_alloc( &Ks[lvl], dim*vx_nr);
-      FUNCTION_FAILURE_HANDLE( err, sparse_alloc, main);
+      err=sparse_flex_alloc( &Ks[lvl], dim*vx_nr, dim*FEINS_SPARSE_COLS_P_ROW_T2);
+      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, main);
       err=projector1_alloc( &P, dim*vx_nr );
       FUNCTION_FAILURE_HANDLE( err, projector1_alloc, main);
       err=vector_alloc( &rhs, dim*vx_nr );
@@ -443,8 +458,8 @@ int main(int argc, char *argv[])
     }
     
   /* allocate memory for matrix Ks and the vectors */
-  err=sparse_flex_alloc( &Ks[lvlm], dim*vx_nr);
-  FUNCTION_FAILURE_HANDLE( err, sparse_alloc, main);
+  err=sparse_flex_alloc( &Ks[lvlm], dim*vx_nr, dim*FEINS_SPARSE_COLS_P_ROW_T2);
+  FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, main);
   err=projector1_alloc( &P, dim*vx_nr );
   FUNCTION_FAILURE_HANDLE( err, projector1_alloc, main);
   err=vector_alloc( &vx_old, dim*vx_nr );
@@ -891,9 +906,9 @@ msh2.vertex[msh2.elhier[i*msh2.eh_w+MCT2EHCHL1+j]*msh2.vx_w+MCT2VXSTRT+1]);
 
   if(set.write_mesh!=0) /* write data for visualisation */
     {
-      err=mesh_write_solution_vtk_t2( &msh2, &nodem, dim, NULL, 0,
+      err=mesh_write_solution_vtk_tx( &msh2, &nodem, dim, NULL, 0,
                               16, "visual/Lame_sol" );
-      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			    main);
     }   
     
