@@ -106,6 +106,22 @@ int main(int argc, char *argv[])
   res_start = -1.0;
   stiffening = 1;  /* ==1 for real Lame', 
                       ==0 for stiffening of small elements */
+
+
+#ifdef HAVE_OPENMP
+#pragma omp parallel
+  {
+#pragma omp master
+    {
+      printf("OpenMP num_threads=%d\n",omp_get_num_threads());
+      printf("  should be number of phyisical cores, use environment variable "
+	     " OMP_NUM_THREADS to modify,\n");
+      printf("  e.g. (bash shell): \"export OMP_NUM_THREADS=4\" \n");
+      printf("  e.g. (most other): \"set OMP_NUM_THREADS 4\" \n");
+    }
+  }
+#endif
+
   
   if (argc>=1)
     {
@@ -190,20 +206,24 @@ int main(int argc, char *argv[])
 		 " (%"dFIDX"+1!= %"dFIDX")\n",  msh2.lvl,  lvl);
 	  return FAIL;
         }
-      err=sparse_flex_alloc( &Ks[lvl], dim*vx_nr);
-      FUNCTION_FAILURE_HANDLE( err, sparse_alloc, main);
+      err=sparse_flex_alloc( &Ks[lvl], dim*vx_nr, dim*FEINS_SPARSE_COLS_P_ROW_T2);
+      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, main);
       err=projector1_alloc( &P, dim*vx_nr );
       FUNCTION_FAILURE_HANDLE( err, projector1_alloc, main);
       err=vector_alloc( &rhs, dim*vx_nr );
       FUNCTION_FAILURE_HANDLE( err, vector_alloc, main);
   
-      for (i=0; i<dim*vx_nr; i++) rhs.V[i]=0.0;
+      /* init rhs vector */
+      for (i=0; i<dim*vx_nr; i++) 
+	{
+	  rhs.V[i]=0.0;
+	}
 	  
       /* Assembly stiffness matrix with projector for Dirichlet 
 	 on the entire boundary */
       err=assem_lame_tx_tx( &Ks[lvl], &P, &msh2, &rhs, &nodem,
 			 lambda, mu, stiffening, 2, 2);
-      FUNCTION_FAILURE_HANDLE( err, assem_lame_tx, main);
+      FUNCTION_FAILURE_HANDLE( err, assem_lame_tx_tx, main);
    
       /* solve the equation system */
       if (set.solver==0) /* variant 1: direct solve (good for coarse mesh) */
@@ -256,9 +276,24 @@ int main(int argc, char *argv[])
 		  n_c_bc++;
 		}
             }
-	  err=coarse_mat_set( &Ks[0], n_c_bc, coarse_bc_nodes,
-			      1, cmat );
-	  FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
+	  if (0==1) 
+	    {
+	      /* old cmat: LAPACK or UMFPACK */
+	      err=coarse_mat_set( &Ks[0], n_c_bc, coarse_bc_nodes,
+				  1, cmat );
+	      FUNCTION_FAILURE_HANDLE( err, coarse_mat_set, main );
+	    }
+	  else
+	    {
+	      /* hypre AMG cmat */
+	      coarse_mat_null_def(cmat);
+	      err=sparse_solve_HypreAMG_cmat_set( &Ks[0], 
+						  n_c_bc, coarse_bc_nodes,
+						  1, cmat );
+	      FUNCTION_FAILURE_HANDLE( err, sparse_solve_HypreAMG_cmat_set, main );
+	      (*cmat).nr     = Ks[0].row_nr;
+	      (*cmat).UMFPACKisinit=0;
+	    }
 	  /* free old cmat and add new one */
 	  if (mg.cmat != NULL)
 	    {
@@ -482,9 +517,9 @@ int main(int argc, char *argv[])
 	for (i=0; i<2*vx_nr; i++)
 	  bla.V[i]=0.0;
 
-	err=mesh_write_solution_vtk_t2( &msh2, &bla, dim, NULL, 0,
+	err=mesh_write_solution_vtk_tx( &msh2, &bla, dim, NULL, 0,
 					19, "visual/mesh_reorder" );
-	FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+	FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 				 main);
 
 	vector_free( &bla );
@@ -507,8 +542,8 @@ int main(int argc, char *argv[])
       /* reached finest level because max levels exhausted,
          need to solve once more */
       /* allocate memory for matrix Ks, P and the vectors */
-      err=sparse_flex_alloc( &Ks[lvlm], dim*vx_nr);
-      FUNCTION_FAILURE_HANDLE( err, sparse_alloc, main);
+      err=sparse_flex_alloc( &Ks[lvlm], dim*vx_nr, dim*FEINS_SPARSE_COLS_P_ROW_T2);
+      FUNCTION_FAILURE_HANDLE( err, sparse_flex_alloc, main);
       err=projector1_alloc( &P, dim*vx_nr );
       FUNCTION_FAILURE_HANDLE( err, projector1_alloc, main);
       err=vector_alloc( &rhs, dim*vx_nr );
@@ -676,9 +711,9 @@ int main(int argc, char *argv[])
 
   if(set.write_mesh!=0) /* write data for visualisation */
     {
-      err=mesh_write_solution_vtk_t2( &msh2, &nodem, dim, NULL, 0,
+      err=mesh_write_solution_vtk_tx( &msh2, &nodem, dim, NULL, 0,
                               16, "visual/Lame_sol" );
-      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			    main);
       err=mesh_write_mesh_gnuplot_tx( &msh2, 17, "visual/Lame_mesh",2 );
       FUNCTION_FAILURE_HANDLE( err, mesh_write_mesh_gnuplot_tx,
@@ -764,9 +799,9 @@ int main(int argc, char *argv[])
 
   if(set.write_mesh!=0) /* write adjoint for visualisation */
     {
-      err=mesh_write_solution_vtk_t2( &msh2, &psi, dim, NULL, 0,
+      err=mesh_write_solution_vtk_tx( &msh2, &psi, dim, NULL, 0,
                               16, "visual/Lame_psi" );
-      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			    main);
     }   
 
@@ -777,9 +812,9 @@ int main(int argc, char *argv[])
 
   if(set.write_mesh!=0) /* write mesh-gradient for visualisation */
     {
-      err=mesh_write_solution_vtk_t2( &msh2, &dIdx, dim, NULL, 0,
+      err=mesh_write_solution_vtk_tx( &msh2, &dIdx, dim, NULL, 0,
                               17, "visual/Lame_DIDX" );
-      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			    main);
     }   
 
@@ -854,9 +889,9 @@ int main(int argc, char *argv[])
       FUNCTION_FAILURE_HANDLE( err, lame_Allaire_et_al_2004__formula_8_t2,
 			       main);
 
-      err=mesh_write_solution_vtk_t2( &msh2, &dIdx_allaire, dim, NULL, 0,
+      err=mesh_write_solution_vtk_tx( &msh2, &dIdx_allaire, dim, NULL, 0,
 				      25, "visual/Lame_DIDX_allaire" );
-      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			       main);
 
       err=vector_alloc( &diff_allaire, dim*vx_nr );
@@ -867,9 +902,9 @@ int main(int argc, char *argv[])
 	  diff_allaire.V[i]=(/*50.0/68.0* */ 1.5*dIdx_allaire.V[i]-dIdx.V[i]);
 	  /*printf("allaire/DAM[%5d] = %16.7e\n", i,dIdx_allaire.V[i]/dIdx.V[i]);*/
 	}
-      err=mesh_write_solution_vtk_t2( &msh2, &diff_allaire, dim, NULL, 0,
+      err=mesh_write_solution_vtk_tx( &msh2, &diff_allaire, dim, NULL, 0,
 				      30, "visual/Lame_DIDX_diff_allaire" );
-      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 			       main);
 
       if (0==1)
@@ -932,19 +967,19 @@ int main(int argc, char *argv[])
 	    }
 
 	  /* write those to files */
-	  err=mesh_write_solution_vtk_t2( &msh2, &dIdx, dim, NULL, 0,
+	  err=mesh_write_solution_vtk_tx( &msh2, &dIdx, dim, NULL, 0,
 					  26, "visual/Lame_coarseBD_DIDX" );
-	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 				   main);
 
-	  err=mesh_write_solution_vtk_t2( &msh2, &dIdx_allaire, dim, NULL, 0,
+	  err=mesh_write_solution_vtk_tx( &msh2, &dIdx_allaire, dim, NULL, 0,
 					  34, "visual/Lame_coarseBD_DIDX_allaire" );
-	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 				   main);
 
-	  err=mesh_write_solution_vtk_t2( &msh2, &diff_allaire, dim, NULL, 0,
+	  err=mesh_write_solution_vtk_tx( &msh2, &diff_allaire, dim, NULL, 0,
 					  39, "visual/Lame_coarseBD_DIDX_diff_allaire" );
-	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+	  FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 				   main);
 
 
@@ -991,9 +1026,9 @@ int main(int argc, char *argv[])
 					      &nodem, &sdifference);
       FUNCTION_FAILURE_HANDLE( err, assem_lame_tx_tx, main);
 
-      err=mesh_write_solution_vtk_t2( &msh2, NULL, 0, &sdifference, 1,
+      err=mesh_write_solution_vtk_tx( &msh2, NULL, 0, &sdifference, 1,
 				      31, "visual/Lame_eppler_sdifference" );
-      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_t2,
+      FUNCTION_FAILURE_HANDLE( err, mesh_write_solution_vtk_tx,
 				   main);
 
       err=mesh_gnuplot_boundary_circles_t2( 2, Rs, 0.01, &msh2, &sdifference, 1, 1, 
